#! /usr/bin/env python3
#
# Copyright (C) 2018  Fx Bricks Inc.
# This file is part of the pfxbrick python module.
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation
# files (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#
# PFx Brick data helpers

import zlib

import pfxbrick.pfxdict as pd
from pfxbrick.pfx import *


def set_with_bit(byte, mask):
    if (byte & mask) == mask:
        return True
    else:
        return False


def get_status_str(x):
    s = ""
    if x in pd.status_dict:
        s = pd.status_dict[x]
    return s


def get_error_str(x):
    s = "None"
    if x in pd.err_dict:
        s = pd.err_dict[x]
    return s


def pprint_bytes(x, address=None):
    def append_ascii(x, a, b):
        s = []
        for i in range(a, b):
            if i < len(x):
                ch = "." if x[i] > 127 else x[i]
                ch = "." if x[i] < 32 else x[i]
                s.append("%c" % (ch))
        return "".join(s)

    s = []
    if address is not None:
        if isinstance(address, str):
            a = int(address, 16)
        else:
            a = address
        ads = "%06X " % (a)
    else:
        ads = ""
    s.append("%s " % (ads))
    for i, b in enumerate(x):
        s.append("%02X " % (b))
        if (i + 1) % 8 == 0 and i > 0:
            s.append(" ")
        if (i + 1) % 16 == 0 and i > 0:
            s.append(" ")
            s.extend(append_ascii(x, i - 15, i + 1))
            if address is not None:
                a += 16
                ads = "%06X " % (a)
            else:
                ads = ""
            s.append("\n")
            s.append("%s " % (ads))
    nb = len(x) % 16
    if nb:
        for i in range(16 - nb + 1):
            s.append("   ")
        s.extend(append_ascii(x, len(x) - nb, len(x) + 1))
    else:
        s.pop()
    print("".join(s))


def listify(x):
    if isinstance(x, list):
        return x
    return [x]


def uint16_toint(bytes):
    res = (int(bytes[0] & 0xFF) << 8) | (int(bytes[1]) & 0xFF)
    return res


def uint32_to_bytes(v):
    x = []
    x.append((v >> 24) & 0xFF)
    x.append((v >> 16) & 0xFF)
    x.append((v >> 8) & 0xFF)
    x.append(v & 0xFF)
    return x


def uint16_to_bytes(v):
    x = []
    x.append((v >> 8) & 0xFF)
    x.append(v & 0xFF)
    return x


def uint32_toint(bytes):
    res = (
        (int(bytes[0] & 0xFF) << 24)
        | (int(bytes[1] & 0xFF) << 16)
        | (int(bytes[2] & 0xFF) << 8)
        | (int(bytes[3]) & 0xFF)
    )
    return res


def uint16_tostr(msb, lsb):
    res = "".join("{:02X}".format(x) for x in [msb, lsb])
    return res


def uint32_tostr(msb, b1, b2, lsb):
    res = "".join("{:02X}".format(x) for x in [msb, b1, b2, lsb])
    return res


def uint16_tover(msb, lsb):
    res = "%02X.%02X" % (msb, lsb)
    return res


def ver_to_bytes(ver):
    major, minor = 0, 0
    vs = ver.split(".")
    major = int(vs[0]) & 0x0F
    vm = "%2s" % (vs[1])
    minor = (int(vm[0]) & 0x0F) << 4
    minor |= int(vm[1]) & 0x0F
    return major, minor


def int8_toint(val):
    if val & 0x80:
        return -((~val & 0x7F) + 1)
    return val & 0x7F


def motor_ch_str(x):
    s = []
    if x & EVT_MOTOR_OUTPUT_MASK:
        s.append("Motor Ch ")
        if x & EVT_MOTOR_OUTPUT_A:
            s.append("A ")
        if x & EVT_MOTOR_OUTPUT_B:
            s.append("B ")
        if x & EVT_MOTOR_OUTPUT_C:
            s.append("C ")
        if x & EVT_MOTOR_OUTPUT_D:
            s.append("D ")
    s = "".join(s)
    return s


def light_ch_str(x):
    s = []
    if x:
        s.append("Ch")
        for i in range(8):
            m = 1 << i
            if x & m:
                s.append(str(i + 1))
    else:
        s.append("None")
    s = " ".join(s)
    return s


def ch_to_mask(ch):
    mask = 0
    if isinstance(ch, list):
        channels = ch
    else:
        channels = [ch]
    for c in channels:
        if c < 1 or c > 8:
            print("Channel out of range")
        else:
            mask = mask | (1 << (c - 1))
    return mask


def address_to_evtch(address):
    evt = (address & EVT_EVENT_ID_MASK) >> 2
    ch = address & EVT_EVENT_CH_MASK
    return evt, ch


def evtch_to_address(evt, ch):
    address = ch & EVT_EVENT_CH_MASK
    address |= (evt << 2) & EVT_EVENT_ID_MASK
    return address


def period_param(period):
    x = float(period)
    if x < 0.250:
        return EVT_PERIOD_100MS
    elif x < 0.50:
        return EVT_PERIOD_250MS
    elif x < 0.75:
        return EVT_PERIOD_500MS
    elif x < 1.0:
        return EVT_PERIOD_750MS
    elif x < 1.25:
        return EVT_PERIOD_1S
    elif x < 1.50:
        return EVT_PERIOD_1_25S
    elif x < 1.75:
        return EVT_PERIOD_1_5S
    elif x < 2.0:
        return EVT_PERIOD_1_75S
    elif x < 2.5:
        return EVT_PERIOD_2S
    elif x < 3.0:
        return EVT_PERIOD_2_5S
    elif x < 4.0:
        return EVT_PERIOD_3S
    elif x < 5.0:
        return EVT_PERIOD_4S
    elif x < 8.0:
        return EVT_PERIOD_5S
    elif x < 10.0:
        return EVT_PERIOD_8S
    elif x < 20.0:
        return EVT_PERIOD_10S
    else:
        return EVT_PERIOD_20S


def period2_param(period):
    x = float(period)
    if x < 0.1:
        return EVT_PERIOD2_50MS
    elif x < 0.2:
        return EVT_PERIOD2_100MS
    elif x < 0.3:
        return EVT_PERIOD2_200MS
    elif x < 0.4:
        return EVT_PERIOD2_300MS
    elif x < 0.5:
        return EVT_PERIOD2_400MS
    elif x < 0.6:
        return EVT_PERIOD2_500MS
    elif x < 0.7:
        return EVT_PERIOD2_600MS
    elif x < 0.8:
        return EVT_PERIOD2_700MS
    elif x < 0.9:
        return EVT_PERIOD2_800MS
    elif x < 1.0:
        return EVT_PERIOD2_900MS
    elif x < 1.25:
        return EVT_PERIOD2_1S
    elif x < 1.5:
        return EVT_PERIOD2_1_25S
    elif x < 1.75:
        return EVT_PERIOD2_1_5S
    elif x < 2.0:
        return EVT_PERIOD2_1_75S
    elif x < 3.0:
        return EVT_PERIOD2_2S
    else:
        return EVT_PERIOD2_3S


def duty_cycle_param(duty_cycle):
    x = float(duty_cycle)
    if x < 0.02:
        return EVT_DUTYCY_1
    elif x < 0.05:
        return EVT_DUTYCY_2
    elif x < 0.1:
        return EVT_DUTYCY_5
    elif x < 0.15:
        return EVT_DUTYCY_10
    elif x < 0.2:
        return EVT_DUTYCY_15
    elif x < 0.25:
        return EVT_DUTYCY_20
    elif x < 0.3:
        return EVT_DUTYCY_25
    elif x < 0.4:
        return EVT_DUTYCY_30
    elif x < 0.5:
        return EVT_DUTYCY_40
    elif x < 0.6:
        return EVT_DUTYCY_50
    elif x < 0.7:
        return EVT_DUTYCY_60
    elif x < 0.75:
        return EVT_DUTYCY_70
    elif x < 0.8:
        return EVT_DUTYCY_75
    elif x < 0.85:
        return EVT_DUTYCY_80
    elif x < 0.9:
        return EVT_DUTYCY_85
    elif x < 0.95:
        return EVT_DUTYCY_90
    elif x < 0.98:
        return EVT_DUTYCY_95
    elif x < 0.99:
        return EVT_DUTYCY_98
    else:
        return EVT_DUTYCY_99


def duration_to_fixed_value(duration):
    x = float(duration)
    if x < 1.0:
        return EVT_SOUND_DUR_500MS
    elif x < 1.5:
        return EVT_SOUND_DUR_1S
    elif x < 2.0:
        return EVT_SOUND_DUR_1_5S
    elif x < 3.0:
        return EVT_SOUND_DUR_2S
    elif x < 4.0:
        return EVT_SOUND_DUR_3S
    elif x < 5.0:
        return EVT_SOUND_DUR_4S
    elif x < 10.0:
        return EVT_SOUND_DUR_5S
    elif x < 15.0:
        return EVT_SOUND_DUR_10S
    elif x < 20.0:
        return EVT_SOUND_DUR_15S
    elif x < 30.0:
        return EVT_SOUND_DUR_20S
    elif x < 45.0:
        return EVT_SOUND_DUR_30S
    elif x < 60.0:
        return EVT_SOUND_DUR_45S
    elif x < 90.0:
        return EVT_SOUND_DUR_60S
    elif x < 120.0:
        return EVT_SOUND_DUR_90S
    elif x < 300.0:
        return EVT_SOUND_DUR_2M
    else:
        return EVT_SOUND_DUR_5M


def printProgressBar(
    iteration, total, prefix="", suffix="", decimals=1, length=100, fill="█"
):
    """
    Call in a loop to create terminal progress bar
    @params:
        iteration   - Required  : current iteration (Int)
        total       - Required  : total iterations (Int)
        prefix      - Optional  : prefix string (Str)
        suffix      - Optional  : suffix string (Str)
        decimals    - Optional  : positive number of decimals in percent complete (Int)
        length      - Optional  : character length of bar (Int)
        fill        - Optional  : bar fill character (Str)
    """
    percent = ("{0:." + str(decimals) + "f}").format(100 * (iteration / float(total)))
    filledLength = int(length * iteration // total)
    bar = fill * filledLength + "-" * (length - filledLength)
    print("\r%s |%s| %s%% %s" % (prefix, bar, percent, suffix), end="\r")
    # Print New Line on Complete
    if iteration == total:
        print()


def get_file_crc32(fn):
    """Returns the CRC32 over the bytes of specified file on the local file system."""
    with open(fn, "rb") as fp:
        fb = fp.read()
    return zlib.crc32(fb) & 0xFFFFFFFF


def bounds_from_notchcount(count):
    """Returns the boundaries between a desired number of power notch levels.

    The power range between 0 to 255 can be subdivided into a desired number
    of notch levels (up to 8 levels).  For example, 2 notch levels would
    return a list with one bound of 128.  4 levels would return a list of
    [64, 128, 192], etc.
    """
    bounds = [round((x + 1) / count * 255) for x in range(count - 1)]
    return bounds


def notch_ranges_from_bounds(bounds):
    """Returns a list of tuples containing the min, mid, max power levels
    based on a list of notch boundary values.
    """
    ranges = []
    count = len(bounds) + 1
    for i in range(count):
        if i == 0:
            mid_speed = bounds[0] / 2
            lower, upper = 0, bounds[0]
        elif i == count - 1:
            mid_speed = bounds[i - 1] + (255 - bounds[i - 1]) / 2
            lower, upper = bounds[i - 1], 255
        else:
            mid_speed = bounds[i - 1] + (bounds[i] - bounds[i - 1]) / 2
            lower, upper = bounds[i - 1], bounds[i]
        ranges.append((lower, round(mid_speed), upper))
    return ranges


def notch_from_speed(speed, bounds):
    """Returns the notch index for a given speed value within a specified list of bounds."""
    ranges = notch_ranges_from_bounds(bounds)
    for i, r in enumerate(ranges):
        if speed >= r[0] and speed < r[2]:
            return i


def safe_unicode_str(text):
    """Trys to decode a byte stream as UTF-8 with a safe fallback to raw ascii."""
    try:
        sn = bytes(text).decode("utf-8")
    except UnicodeDecodeError:
        sn = []
        for b in text:
            if b >= 0x20 and b <= 127:
                sn.append(str(b))
        sn = "".join(sn)
    sn = sn.rstrip("\0")
    return sn


def is_version_less_than(ver, other):
    """ "Determines if a version number is less than another version number string"""
    sv = ver.split(".")
    ov = other.split(".")
    sv_major = int(sv[0])
    if len(sv) > 1:
        sv_minor = int(sv[1])
    ov_major = int(ov[0])
    if len(ov) > 1:
        ov_minor = int(ov[1])
    if sv_major < ov_major:
        return True
    if sv_major == ov_major:
        if sv_minor < ov_minor:
            return True
    return False


def pprint_diff(x, y, address=None):
    from rich import print

    def append_ascii(x, a, b):
        s = []
        for i in range(a, b):
            if i < len(x):
                ch = "." if x[i] > 127 else x[i]
                ch = "." if x[i] < 32 else x[i]
                s.append("%c" % (ch))
        return "".join(s)

    s = []
    if address is not None:
        if isinstance(address, str):
            a = int(address, 16)
        else:
            a = address
        ads = "%06X " % (a)
    else:
        ads = ""
    s.append("%s " % (ads))
    for i, (b, by) in enumerate(zip(x, y)):
        if b == by:
            s.append("[bold black]%02X|%02X[/] " % (b, by))
        else:
            s.append("[bold red]%02X|%02X[/] " % (b, by))

        if (i + 1) % 8 == 0 and i > 0:
            s.append(" ")
        if (i + 1) % 16 == 0 and i > 0:
            s.append(" ")
            s.extend(append_ascii(x, i - 15, i + 1))
            if address is not None:
                a += 16
                ads = "%06X " % (a)
            else:
                ads = ""
            s.append("\n")
            s.append("%s " % (ads))
    nb = len(x) % 16
    if nb:
        for i in range(16 - nb + 1):
            s.append("   ")
        s.extend(append_ascii(x, len(x) - nb, len(x) + 1))
    else:
        s.pop()
    print("".join(s))


def get_one_pfxbrick(serial_no=None):
    """Returns only one instance of a PFx Brick if one or more are connected.
    If only one is connected, then a PFxBrick object is returned for it.
    If more than one is connected, a warning to select using serial number is shown.
    """
    from pfxbrick.pfxbrick import PFxBrick, find_bricks

    b = None
    bricks = find_bricks()
    if len(bricks) > 1 and serial_no is None:
        print(
            "More than one PFx Brick is attached.  Please specify brick serial number with the -s argument."
        )
        print("Currently attached PFx Bricks:")
        for brick in bricks:
            b = PFxBrick(brick)
            r = b.open()
            b.get_status()
            name = b.get_name()
            print(
                "[light_slate_blue]%-4s[/] [bold cyan]%-24s[/] Serial no: [bold cyan]%-9s[/] Name: [bold yellow]%s[/]"
                % (b.product_id, b.product_desc, b.serial_no, name)
            )
            b.close()
        exit()
    if serial_no is not None and len(bricks) > 1:
        b = PFxBrick(serial_no=serial_no)
    else:
        b = PFxBrick()
    return b


def shorter_str(s):
    """Replaces words in a string that are found in the helper dictionaries
    in pfxdict.py  with shorter versions for a more compact description."""
    tokens = [
        "Increase Inc",
        "Decrease Dec",
        "Emergency Emcy",
        "Oscillate Osc",
        "Direction Dir",
        "Brightness Bright",
        "positive +",
        "negative -",
        "Modulated Mod",
        "Alternating Alt",
        "Style ",
        "Left L",
        "Right R",
        "Joystick Joy",
        "Event Evt",
        "Centre Ctr",
        "Flashers Flash",
    ]
    for t in tokens:
        ts = t.split()
        if len(ts) < 2:
            s = s.replace(ts[0], "")
        else:
            s = s.replace(ts[0], ts[1])
    return s



#! /usr/bin/env python3
"""
pfxrun - run a script file on the PFx Brick
"""
import argparse

from pfxbrick import *


def main():
    parser = argparse.ArgumentParser(
        description="Run a script file on the PFx Brick",
        prefix_chars="-+",
    )
    parser.add_argument(
        "file", metavar="file", type=str, help="file name or file ID of script"
    )
    parser.add_argument(
        "-e",
        "--end",
        action="store_true",
        default=False,
        help="end script execution",
    )
    parser.add_argument(
        "-s",
        "--serialno",
        default=None,
        help="Specify PFx Brick with serial number (if more than one connected)",
    )
    args = parser.parse_args()
    argsd = vars(args)

    b = get_one_pfxbrick(argsd["serialno"])
    r = b.open()
    if not r:
        exit()
    b.refresh_file_dir()
    if argsd["end"]:
        b.stop_script()
        print("Script execution stopped")
        exit()
    f = str(argsd["file"])
    if f.isnumeric():
        f = int(argsd["file"])
    fid = b.file_id_from_str_or_int(f)
    if fid < 0xFF:
        b.run_script(fid)
        print("Running script file %s" % (argsd["file"]))
    else:
        print("File %s is not found on the PFx Brick" % (argsd["file"]))
    b.close()


if __name__ == "__main__":
    main()



#! /usr/bin/env python3
#
# Copyright (C) 2018  Fx Bricks Inc.
# This file is part of the pfxbrick python module.
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation
# files (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#
# PFx Brick configuration data helpers

import pfxbrick.pfxdict as pd
from pfxbrick.pfx import *
from pfxbrick.pfxhelpers import *


class PFxAction:
    """
    Action data structure class.

    This class reflects the 16 byte data structure used internally
    by the PFx Brick to execute a composite action of motor, lighting,
    and sound effects.

    Attributes:
        command (:obj:`int`): Command byte

        motorActionId (:obj:`int`): Motor action ID and motor channel mask byte

        motorParam1 (:obj:`int`): Motor parameter 1

        motorParam2 (:obj:`int`): Motor parameter 2

        lightFxId (:obj:`int`): Light Fx ID byte

        lightOutputMask (:obj:`int`): Light channel output mask

        lightPFOutputMask (:obj:`int`): Light channel on PF output mask

        lightParam1 (:obj:`int`): Lighting parameter 1

        lightParam2 (:obj:`int`): Lighting parameter 2

        lightParam3 (:obj:`int`): Lighting parameter 3

        lightParam4 (:obj:`int`): Lighting parameter 4

        lightParam5 (:obj:`int`): Lighting parameter 5

        soundFxId (:obj:`int`): Sound Fx ID byte

        soundFileId (:obj:`int`): Sound file ID

        soundParam1 (:obj:`int`): Sound parameter 1

        soundParam2 (:obj:`int`): Sound parameter 2

    """

    def __init__(self):
        self.command = 0
        self.motorActionId = 0
        self.motorParam1 = 0
        self.motorParam2 = 0
        self.lightFxId = 0
        self.lightOutputMask = 0
        self.lightPFOutputMask = 0
        self.lightParam1 = 0
        self.lightParam2 = 0
        self.lightParam3 = 0
        self.lightParam4 = 0
        self.lightParam5 = 0
        self.soundFxId = 0
        self.soundFileId = 0
        self.soundParam1 = 0
        self.soundParam2 = 0

    def __eq__(self, other):
        """Dunder method for equality"""
        for k, v in self.__dict__.items():
            if k not in other.__dict__:
                return False
            if not v == other.__dict__[k]:
                return False
        return True

    def all_off(self):
        """
        Populates an action to turn off all motors, lights, and sound.
        """
        self.clear()
        self.command = EVT_COMMAND_ALL_OFF
        return self

    def set_motor_speed(self, ch, speed, duration=None):
        """
        Populates an action to set the speed of specified motor channel(s).
        The motor speed is specified between -100% and +100% where negative
        values are in the reverse direction and positive values are in
        the forward direction.

        :param ch: [:obj:`int`] a list of motor channels (1-4)
        :param speed: :obj:`int` desired motor speed (-100 to +100)
        :param duration: :obj:`float` optional duration (in seconds) to run motor, runs indefinitely if not specified
        :returns: :obj:`PFxAction` self

        If the duration value is specified, it represents the desired motor
        run time in seconds. Note that this value will be rounded to the
        nearest fixed interval of the DURATION parameter as defined in the ICD
        ranging between 16 fixed values from 0.5 sec to 5 min.
        """
        sf = float(speed)
        if sf > 100.0:
            sf = 100.0
        if sf < -100.0:
            sf = -100.0
        sf = (sf / 100.0) * 63.0
        s = int(abs(sf)) & EVT_MOTOR_SPEED_HIRES_MASK
        s |= EVT_MOTOR_SPEED_HIRES
        if speed < 0:
            s |= EVT_MOTOR_SPEED_HIRES_REV
        self.motorParam1 = s

        m = ch_to_mask(listify(ch)) & EVT_MOTOR_OUTPUT_MASK
        if duration is not None:
            m |= EVT_MOTOR_SET_SPD_TIMED
            self.motorParam2 = duration_to_fixed_value(duration)
        else:
            m |= EVT_MOTOR_SET_SPD
        self.motorActionId = m
        return self

    def stop_motor(self, ch):
        """
        Populates an action to stop the specifed motor channel(s).

        :param ch: [:obj:`int`] a list of motor channels (1-4)
        :returns: :obj:`PFxAction` self
        """
        m = ch_to_mask(listify(ch)) & EVT_MOTOR_OUTPUT_MASK
        m |= EVT_MOTOR_ESTOP
        self.motorActionId = m
        return self

    def light_on(self, ch):
        """
        Populates an action to turn on selected light outputs.

        :param ch: [:obj:`int`] a list of light channels (1-8)
        :returns: :obj:`PFxAction` self
        """
        self.lightOutputMask = ch_to_mask(listify(ch))
        self.lightFxId = EVT_LIGHTFX_ON_OFF_TOGGLE
        self.lightParam4 = EVT_TRANSITION_ON
        return self

    def light_off(self, ch):
        """
        Populates an action to turn off selected light outputs.

        :param ch: [:obj:`int`] a list of light channels (1-8)
        :returns: :obj:`PFxAction` self
        """
        self.lightOutputMask = ch_to_mask(listify(ch))
        self.lightFxId = EVT_LIGHTFX_ON_OFF_TOGGLE
        self.lightParam4 = EVT_TRANSITION_OFF
        return self

    def light_toggle(self, ch):
        """
        Populates an action to toggle the state of selected light outputs.

        :param ch: [:obj:`int`] a list of light channels (1-8)
        :returns: :obj:`PFxAction` self
        """
        self.lightOutputMask = ch_to_mask(listify(ch))
        self.lightFxId = EVT_LIGHTFX_ON_OFF_TOGGLE
        self.lightParam4 = EVT_TRANSITION_TOGGLE
        return self

    def set_brightness(self, ch, brightness):
        """
        Populates an action to set the brightness of selected light outputs.

        :param ch: [:obj:`int`] a list of light channels (1-8)
        :param brightness: :obj:`int` brightness (0 - 255 max)
        :returns: :obj:`PFxAction` self
        """
        x = brightness
        if x > 255:
            x = 255
        if x < 0:
            x = 0
        self.lightOutputMask = ch_to_mask(listify(ch))
        self.lightFxId = EVT_LIGHTFX_SET_BRIGHT
        self.lightParam1 = x
        return self

    def combo_light_fx(self, fx, param=[0, 0, 0, 0, 0]):
        """
        Populates an action with a user specified combination light effect
        and associated parameters.

        :param fx: :obj:`int` desired light effect
        :param param: [:obj:`int`] a list of up to 5 light parameters
        :returns: :obj:`PFxAction` self
        """
        return self.light_fx([], fx | EVT_LIGHT_COMBO_MASK, param)

    def light_fx(self, ch, fx, param=[0, 0, 0, 0, 0]):
        """
        Populates an action with a user specified light effect and
        associated parameters.

        :param ch: [:obj:`int`] a list of light channels (1-8)
        :param fx: :obj:`int` desired light effect
        :param param: [:obj:`int`] a list of up to 5 light parameters
        :returns: :obj:`PFxAction` self

        The details of specifying the light **fx** and **param** items
        is described in detail in the ICD document. The **pfx.py**
        file contains convenient pre-defined constants for all of
        the light effect types and parameter values.

        An example of using this method is as follows::

            p = [EVT_PERIOD_1S, EVT_DUTYCY_10, EVT_BURST_COUNT_2, EVT_TRANSITION_TOGGLE]
            a = PFxAction().light_fx([1,4], EVT_LIGHTFX_STROBE_P, p)

        This specifies a strobe light effect on channels 1 and 4 with
        a 1 second period, 10% duty cycle, two light pulses and with
        a toggle activation.
        """
        self.lightOutputMask = ch_to_mask(listify(ch))
        self.lightFxId = fx
        for i, p in enumerate(param):
            if i == 0:
                self.lightParam1 = p
            elif i == 1:
                self.lightParam2 = p
            elif i == 2:
                self.lightParam3 = p
            elif i == 3:
                self.lightParam4 = p
            elif i == 4:
                self.lightParam5 = p
        return self

    def sound_fx(self, fx, param=[0, 0], fileID=None):
        """
        Populates an action with a user specified sound effect and
        associated parameters.

        :param fx: :obj:`int` desired sound action
        :param param: [:obj:`int`] a list of up to 2 sound parameters
        :param fileID: :obj:`int` file ID of an audio file in the file system
        :returns: :obj:`PFxAction` self

        The details of specifying the sound **fx** and **param** items
        is described in detail in the ICD document. The **pfx.py**
        file contains convenient pre-defined constants for all of
        the sound effect types and parameter values.

        An example of using this method is as follows::

            p = [EVT_SOUND_DUR_10S]
            a = PFxAction().sound_fx(EVT_SOUND_PLAY_DUR, p, 5)

        This specifies an action to playback an audio file with ID=5
        for a fixed duration of 10 seconds.
        """
        self.soundFxId = fx
        if fileID is not None:
            self.soundFileId = fileID
        for i, p in enumerate(param):
            if i == 0:
                self.soundParam1 = p
            elif i == 1:
                self.soundParam2 = p
        return self

    def play_audio_file(self, fileID):
        """
        Populates an action to play an audio file once.

        :param fileID: :obj:`int` file ID of an audio file in the file system
        :returns: :obj:`PFxAction` self

        This is a convenience wrapper for the sound_fx method.
        """
        return self.sound_fx(EVT_SOUND_PLAY_ONCE, [EVT_SOUND_TOGGLE], fileID)

    def stop_audio_file(self, fileID):
        """
        Populates an action to stop playback of an audio file.

        :param fileID: :obj:`int` file ID of an audio file in the file system
        :returns: :obj:`PFxAction` self

        This is a convenience wrapper for the sound_fx method.
        """
        return self.sound_fx(EVT_SOUND_STOP, [], fileID)

    def repeat_audio_file(self, fileID):
        """
        Populates an action for repeated playback of an audio file.

        :param fileID: :obj:`int` file ID of an audio file in the file system
        :returns: :obj:`PFxAction` self

        This is a convenience wrapper for the sound_fx method.
        """
        return self.sound_fx(EVT_SOUND_PLAY_CONT, [], fileID)

    def set_volume(self, volume):
        """
        Populates an action to set the audio volume.

        :param volume: :obj:`int` desired audio volume (0 - 100%)
        :returns: :obj:`PFxAction` self

        This is a convenience wrapper for the sound_fx method.
        """
        vf = float(volume)
        if vf > 100.0:
            vf = 100.0
        if vf < 0.0:
            vf = 0.0
        vf = (vf / 100.0) * 255.0
        v = int(vf)
        return self.sound_fx(EVT_SOUND_SET_VOL, [0, v])

    def is_empty(self):
        """Determines if the action is clear/undefined"""
        if self.command > 0:
            return False
        if self.motorActionId > 0:
            return False
        if self.motorParam1 > 0:
            return False
        if self.motorParam2 > 0:
            return False
        if self.lightFxId > 0:
            return False
        if self.lightOutputMask > 0:
            return False
        if self.lightPFOutputMask > 0:
            return False
        if self.lightParam1 > 0:
            return False
        if self.lightParam2 > 0:
            return False
        if self.lightParam3 > 0:
            return False
        if self.lightParam4 > 0:
            return False
        if self.lightParam5 > 0:
            return False
        if self.soundFxId > 0:
            return False
        if self.soundFileId > 0:
            return False
        if self.soundParam1 > 0:
            return False
        if self.soundParam2 > 0:
            return False
        return True

    def clear(self):
        """
        Sets all the action data in this class to zero.
        """
        self.command = 0
        self.motorActionId = 0
        self.motorParam1 = 0
        self.motorParam2 = 0
        self.lightFxId = 0
        self.lightOutputMask = 0
        self.lightPFOutputMask = 0
        self.lightParam1 = 0
        self.lightParam2 = 0
        self.lightParam3 = 0
        self.lightParam4 = 0
        self.lightParam5 = 0
        self.soundFxId = 0
        self.soundFileId = 0
        self.soundParam1 = 0
        self.soundParam2 = 0

    def from_bytes(self, msg):
        """
        Converts the message string bytes read from the PFx Brick into
        the corresponding data members of this class.
        """
        self.command = msg[1]
        self.motorActionId = msg[2]
        self.motorParam1 = msg[3]
        self.motorParam2 = msg[4]
        self.lightFxId = msg[5]
        self.lightOutputMask = msg[6]
        self.lightPFOutputMask = msg[7]
        self.lightParam1 = msg[8]
        self.lightParam2 = msg[9]
        self.lightParam3 = msg[10]
        self.lightParam4 = msg[11]
        self.lightParam5 = msg[12]
        self.soundFxId = msg[13]
        self.soundFileId = msg[14]
        self.soundParam1 = msg[15]
        self.soundParam2 = msg[16]

    def to_bytes(self):
        """
        Converts the data members of this class to the message
        string bytes which can be sent to the PFx Brick.
        """
        msg = []
        msg.append(self.command)
        msg.append(self.motorActionId)
        msg.append(self.motorParam1)
        msg.append(self.motorParam2)
        msg.append(self.lightFxId)
        msg.append(self.lightOutputMask)
        msg.append(self.lightPFOutputMask)
        msg.append(self.lightParam1)
        msg.append(self.lightParam2)
        msg.append(self.lightParam3)
        msg.append(self.lightParam4)
        msg.append(self.lightParam5)
        msg.append(self.soundFxId)
        msg.append(self.soundFileId)
        msg.append(self.soundParam1)
        msg.append(self.soundParam2)
        return msg

    def verbose_line_str(self, brick):
        """
        Human readable string of action object in single line.

        :param brick: :obj:`PFxBrick` reference to a PFx Brick in order to query filename
        """
        s = []
        if self.command > 0:
            s.append("Cmd: %s" % shorter_str(pd.command_dict[self.command]))
        if self.motorActionId & EVT_MOTOR_OUTPUT_MASK:
            x = pd.motor_action_dict[self.motorActionId & EVT_MOTOR_ACTION_ID_MASK]
            s.append("%s%s" % (motor_ch_str(self.motorActionId), shorter_str(x)))
        if self.lightFxId & EVT_LIGHT_COMBO_MASK:
            sf = pd.combo_lightfx_dict[self.lightFxId & EVT_LIGHT_ID_MASK]
            s.append("Combo light: %s" % (shorter_str(sf)))
        elif self.lightFxId & EVT_LIGHT_ID_MASK:
            sf = pd.ind_lightfx_dict[self.lightFxId & EVT_LIGHT_ID_MASK]
            sc = light_ch_str(self.lightOutputMask)
            s.append("Light %s %s" % (sc, sf))
        if self.soundFxId:
            x = pd.soundfx_dict[self.soundFxId]
            if self.soundFileId:
                fn = brick.filedir.get_filename(self.soundFileId)
                s.append('Sound %s "%s" (%d)' % (x, fn, self.soundFileId))
            else:
                s.append("Sound %s" % (x))
        return " ".join(s)

    def __str__(self):
        """
        Convenient human readable string of the action data structure. This allows
        a :py:class:`PFxAction` object to be used with :obj:`str` and :obj:`print` methods.
        """
        sb = []
        sb.append(
            "Command           : [%02X] %s"
            % (self.command, pd.command_dict[self.command])
        )
        if self.motorActionId & EVT_MOTOR_OUTPUT_MASK == 0:
            sb.append("Motor Action ID   : [%02X] None" % (self.motorActionId))
        else:
            sb.append(
                "Motor Action ID   : [%02X] %s %s"
                % (
                    self.motorActionId,
                    pd.motor_action_dict[self.motorActionId & EVT_MOTOR_ACTION_ID_MASK],
                    motor_ch_str(self.motorActionId),
                )
            )
        sb.append("Motor Param 1     : [%02X]" % (self.motorParam1))
        sb.append("Motor Param 2     : [%02X]" % (self.motorParam2))
        sf = ""
        if self.lightFxId & EVT_LIGHT_COMBO_MASK:
            sf = pd.combo_lightfx_dict[self.lightFxId & EVT_LIGHT_ID_MASK]
        else:
            sf = pd.ind_lightfx_dict[self.lightFxId & EVT_LIGHT_ID_MASK]
        sb.append("Light Fx ID       : [%02X] %s" % (self.lightFxId, sf))
        sb.append(
            "Light Output Mask : [%02X] %s"
            % (self.lightOutputMask, light_ch_str(self.lightOutputMask))
        )
        sb.append("Light PF Out Mask : [%02X]" % (self.lightPFOutputMask))
        sb.append("Light Param 1     : [%02X]" % (self.lightParam1))
        sb.append("Light Param 2     : [%02X]" % (self.lightParam2))
        sb.append("Light Param 3     : [%02X]" % (self.lightParam3))
        sb.append("Light Param 4     : [%02X]" % (self.lightParam4))
        sb.append("Light Param 5     : [%02X]" % (self.lightParam5))
        sb.append("Sound Fx ID       : [%02X]" % (self.soundFxId))
        sb.append("Sound File ID     : [%02X]" % (self.soundFileId))
        sb.append("Sound Param 1     : [%02X]" % (self.soundParam1))
        sb.append("Sound Param 2     : [%02X]" % (self.soundParam2))
        s = "\n".join(sb)
        return s



# Generated by h2py from pfx.h
#
# Copyright (C) 2019  Fx Bricks Inc.
# This file is part of the pfxbrick python module.
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation
# files (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#
# /*******************************************************************************
#  *******************************************************************************
#
#  PFXBrick Top Level Definitions
#
#   File Name:
#     pfx.h
#
#   Summary:
#     Defines types and APIs associated with the PFx Brick.
#
#   Description:
#     Defines types and APIs associated with the PFx Brick.
#
#   Copyright (c) 2017  Fx Bricks Inc.
#
#   Jun 3, 2016
#
#   Jun 8, 2016 : Added USB HID messages to support event table definition
#                 for EV3 IR remote beacon:
#                   PFX_USB_CMD_GET_EVENT_CH1_EV3
#                   PFX_USB_CMD_SET_EVENT_CH1_EV3
#                   PFX_USB_CMD_GET_EVENT_CH2_EV3
#                   PFX_USB_CMD_SET_EVENT_CH2_EV3
#                   PFX_USB_CMD_GET_EVENT_CH3_EV3
#                   PFX_USB_CMD_SET_EVENT_CH3_EV3
#                   PFX_USB_CMD_GET_EVENT_CH4_EV3
#                   PFX_USB_CMD_SET_EVENT_CH4_EV3
#                 Added a motor configuration parameter which enables low
#                 freq PWM at low speeds for torque compensation:
#                   PFX_CFG_MOTOR_TRQCOMP
#
#   Jul 28, 2016 : Added new parameters types.  Added new command messages.
#
#   Aug 10, 2016 : Added error code defines
#
#   Aug 12, 2016 : Major redesign based on future-proofing capacity
#
#   Sep 23, 2016 : Added the PFX_USB_CMD_GET_ICD_REV message
#                  Redefined the MOTOR_STOP and MOTOR_COAST definitions
#                  Added the EVT_COMMAND_IR_LOCK_TOGGLE command
#                  Added the PFX_CFG_AUDIO_DRC_MASK to the PFX settings config byte
#
#   Oct 14, 2016 : Changed the PFX_USB_CMD_GET_STATUS message format
#                  Changed the PFX_USB_CMD_GET_CONFIG message format
#                  Other changes to reflect ICD v.3.0
#
#   Oct 24, 2016 : Changed to reflect ICD v.3.14
#
#   Nov 7, 2016  : Changed to reflect ICD v.3.15
#
#   Nov 27, 2016 : Changed to reflect ICD v.3.16
#
#   Dec 16, 2016 : Added new product descriptor for updated PCB
#
#   Dec 18, 2016 : Changed to reflect ICD v.3.17
#
#   Jan 25, 2017 : Changed to reflect ICD v.3.20
#
#   Mar 25, 2017 : Added more constant defines shared between firmware and App
#                  Changes to reflect ICD v.3.22
#
#   May 15, 2017 : Added more product defines and masks
#
#   Jul 4, 2017  : Updated product part number/descriptors
#
#   Aug 31, 2017 : Updated to reflect ICD 3.30
#
#   Oct 12, 2017 : Updated to reflect ICD 3.33
#
#   Dec 12, 2017 : Updated to reflect ICD 3.36
#
#   May 20, 2019 : Updated to reflect ICD 3.37
#
# *******************************************************************************
# *******************************************************************************/

# /*******************************************************************************
#  *******************************************************************************
#
#  PFX Product Descriptors
#
# *******************************************************************************
# *******************************************************************************/
PFX_USB_VENDOR_ID = 0x04D8
PFX_USB_PRODUCT_ID = 0xEF74
PFX_USB_PRODUCT_ID_LEGACY = 0x00F8
PFX_USB_USAGE_PAGE = 0xFF00
PFX_USB_USAGE_HID_1 = 0x0001
PFX_USB_USAGE_HID_2 = 0x0002
PFX_USB_USAGE_HID_3 = 0x0003
PFX_USB_USAGE_HID_4 = 0x0004
PFX_BLE_GATT_DEV_INFO_UUID = 0x180A
PFX_BLE_GATT_DEV_SN_UUID = 0x2A25
PFX_BLE_GATT_UART_UUID = "49535343-FE7D-4AE5-8FA9-9FAFD205E455"
PFX_BLE_GATT_UART_RX_UUID = "49535343-1E4D-4BD9-BA61-23C647249616"
PFX_BLE_GATT_UART_TX_UUID = "49535343-8841-43F4-A8D4-ECBE34729BB3"
PFX_BLE_GATT_UART_TXRESP_UUID = "49535343-A4C8-39B3-2F49-511CFF073B7E"
PFX_PFXBRICK_PROD_MASK = 0x0F00
PFX_PFXBRICK_PROD = 0x0200
PFX_PFXBRICK_BT_MASK = 0x8000
PFX_PFXBRICK_PRODUCTION_MASK = 0x3F00
PFX_PFXBRICK_PRODUCTION_PN = 0x2200
PFX_PFXBRICK_ALPHA_PN = 0x1201
PFX_PFXBRICK_ALPHA_DESC = "PFx Brick alpha"
PFX_PFXBRICK_ALPHA_FLASH_SZ = 0x80000
PFX_PFXBRICK_BETA_PN = 0x1202
PFX_PFXBRICK_BETA_DESC = "PFx Brick beta"
PFX_PFXBRICK_BETA_FLASH_SZ = 0x400000
PFX_PFXBRICK_GAMMA_PN = 0x1203
PFX_PFXBRICK_GAMMA_DESC = "PFx Brick gamma"
PFX_PFXBRICK_GAMMA_FLASH_SZ = 0x400000
PFX_PFXBRICK_DELTA_PN = 0x1204
PFX_PFXBRICK_DELTA_DESC = "PFx Brick delta"
PFX_PFXBRICK_DELTA_FLASH_SZ = 0x400000
PFX_PFXBRICK_DELTA_IR_PN = 0x9204
PFX_PFXBRICK_DELTA_IR_DESC = "PFx Brick delta IR"
PFX_PFXBRICK_DELTA_IR_FLASH_SZ = 0x400000
PFX_PFXBRICK_IR_4MB_PN = 0x2204
PFX_PFXBRICK_IR_4MB_DESC = "PFx Brick IR 4 MB"
PFX_PFXBRICK_IR_4MB_FLASH_SZ = 0x400000
PFX_PFXBRICK_IR_8MB_PN = 0x2208
PFX_PFXBRICK_IR_8MB_DESC = "PFx Brick IR 8 MB"
PFX_PFXBRICK_IR_8MB_FLASH_SZ = 0x800000
PFX_PFXBRICK_IR_16MB_PN = 0x2216
PFX_PFXBRICK_IR_16MB_DESC = "PFx Brick IR 16 MB"
PFX_PFXBRICK_IR_16MB_FLASH_SZ = 0x1000000
PFX_PFXBRICK_4MB_PN = 0xA204
PFX_PFXBRICK_4MB_DESC = "PFx Brick 4 MB"
PFX_PFXBRICK_4MB_FLASH_SZ = 0x400000
PFX_PFXBRICK_8MB_PN = 0xA208
PFX_PFXBRICK_8MB_DESC = "PFx Brick 8 MB"
PFX_PFXBRICK_8MB_FLASH_SZ = 0x800000
PFX_PFXBRICK_16MB_PN = 0xA216
PFX_PFXBRICK_16MB_DESC = "PFx Brick 16 MB"
PFX_PFXBRICK_16MB_FLASH_SZ = 0x1000000
PFX_PFXBRICK_GENERIC_PN = 0x1200
PFX_PFXBRICK_GENERIC_DESC = "PFx Brick"
PFX_PFXBRICK_GENERIC_FLASH_SZ = 0x400000
PFX_PFXLITE_PROD_MASK = 0x0F00
PFX_PFXLITE_PROD = 0x0700
PFX_PFXLITE_ALPHA_PN = 0x1701
PFX_PFXLITE_ALPHA_DESC = "PFx Lite alpha"
PFX_PFXLITE_ALPHA_FLASH_SZ = 0x80000
PFX_PFXLITE_PN = 0x2701
PFX_PFXLITE_DESC = "PFx Lite"
PFX_PFXLITE_FLASH_SZ = 0x80000
PFX_PFXBRICK_PRO_ALPHA_PN = 0x1401
PFX_PFXBRICK_PRO_ALPHA_DESC = "PFx Brick Pro alpha"
PFX_PFXBRICK_PRO_ALPHA_FLASH_SZ = 0x400000
PFX_PFXBRICK_PRO_4MB_PN = 0xA404
PFX_PFXBRICK_PRO_4MB_DESC = "PFx Brick Pro 4 MB"
PFX_PFXBRICK_PRO_4MB_FLASH_SZ = 0x400000
PFX_PFXBRICK_PRO_8MB_PN = 0xA408
PFX_PFXBRICK_PRO_8MB_DESC = "PFx Brick Pro 8 MB"
PFX_PFXBRICK_PRO_8MB_FLASH_SZ = 0x800000
PFX_PFXBRICK_PRO_16MB_PN = 0xA416
PFX_PFXBRICK_PRO_16MB_DESC = "PFx Brick Pro 16 MB"
PFX_PFXBRICK_PRO_16MB_FLASH_SZ = 0x1000000
PFX_PFXBRICK_DCC_ALPHA_PN = 0x1301
PFX_PFXBRICK_DCC_ALPHA_DESC = "PFx Brick DCC alpha"
PFX_PFXBRICK_DCC_PN = 0x2302
PFX_PFXBRICK_DCC_DESC = "PFx Brick DCC"
PFX_PFXBOT_ALPHA_PN = 0x1501
PFX_PFXBOT_ALPHA_DESC = "PFx Bot alpha"
PFX_PFXBOT_PN = 0x2502
PFX_PFXBOT_DESC = "PFx Bot"
PFX_FLASH_SECTOR_SZ = 0x1000

# /*******************************************************************************
#  *******************************************************************************
#
#  Configuration Definitions
#
# *******************************************************************************
# *******************************************************************************/
PFX_NAME_MAX = 24
PFX_VER_MAJOR_MASK = 0xFF00
PFX_VER_MINOR_MASK = 0x00FF
PFX_CFG_AUDIO_DRC_MASK = 0x40
PFX_CFG_LOCK_MODE_MASK = 0x30
PFX_CFG_STATLED_MASK = 0x01
PFX_CFG_VOLBEEP_MASK = 0x02
PFX_CFG_POWERSAVE_MASK = 0x0C
PFX_CFG_STATLED_OFF = 0x01
PFX_CFG_STATLED_ON = 0x00
PFX_CFG_VOLBEEP_ON = 0x02
PFX_CFG_VOLBEEP_OFF = 0x00
PFX_CFG_POWERSAVE_OFF = 0x00
PFX_CFG_POWERSAVE_30M = 0x04
PFX_CFG_POWERSAVE_60M = 0x08
PFX_CFG_POWERSAVE_3HR = 0x0C
PFX_CFG_POWERSAVE_MASK = 0x0C
PFX_CFG_LOCKOUT_INH = 0x00
PFX_CFG_LOCKOUT_CH1 = 0x10
PFX_CFG_LOCKOUT_ALLCH = 0x20
PFX_CFG_AUDIO_DRC_ON = 0x40
PFX_CFG_AUDIO_DRC_OFF = 0x00
PFX_CFG_IRAUTO_OFF_NEVER = 0x0
PFX_CFG_IRAUTO_OFF_1MIN = 0x1
PFX_CFG_IRAUTO_OFF_5MIN = 0x2
PFX_CFG_IRAUTO_OFF_IMMEDIATE = 0x3
PFX_CFG_BLEAUTO_OFF_NEVER = 0x0
PFX_CFG_BLEAUTO_OFF_1MIN = 0x1
PFX_CFG_BLEAUTO_OFF_5MIN = 0x2
PFX_CFG_BLEAUTO_OFF_IMMEDIATE = 0x3
PFX_CFG_BLE_MOTOR_CONTINUE = 0x0
PFX_CFG_BLE_MOTOR_STOP = 0x1
PFX_CFG_BLE_TX_PWR_MAX = 0x0
PFX_CFG_BLE_TX_PWR_MIN = 0x5
PFX_CFG_MOTOR_INVERT = 0x01
PFX_CFG_MOTOR_NORMAL = 0x00
PFX_CFG_MOTOR_TRQCOMP = 0x02
PFX_CFG_MOTOR_TLGMODE = 0x04
PFX_AUDIO_FILES_MAX = 60
PFX_AUDIO_FILENAME_MAX = 32
PFX_AUDIO_CHANNELS = 4
PFX_MOTOR_CHANNELS = 2
PFX_MOTOR_CHANNELS_MAX = 4
PFX_LIGHT_PORTS = 8
PFX_LIGHT_CHANNELS = 10
PFX_LIGHT_CHANNELS_MAX = 12

# /*******************************************************************************
#  *******************************************************************************
#
#  Default Value Definitions
#
# *******************************************************************************
# *******************************************************************************/
PFX_DEFAULT_NAME = "My PFx Brick"
PFX_DEFAULT_STATLED = PFX_CFG_STATLED_ON
PFX_DEFAULT_VOLBEEP = PFX_CFG_VOLBEEP_OFF
PFX_DEFAULT_POWERSAVE = PFX_CFG_POWERSAVE_OFF
PFX_DEFAULT_LOCKOUT = PFX_CFG_LOCKOUT_INH
PFX_DEFAULT_AUDIO_DRC = PFX_CFG_AUDIO_DRC_OFF
PFX_DEFAULT_AUDIO_BASS = 0
PFX_DEFAULT_AUDIO_TREBLE = 0
PFX_DEFAULT_BRIGHTNESS = 0xC0
PFX_DEFAULT_VOLUME = 0xA0
PFX_DEFAULT_IRAUTO_OFF = PFX_CFG_IRAUTO_OFF_NEVER
PFX_DEFAULT_BLEAUTO_OFF = PFX_CFG_BLEAUTO_OFF_NEVER
PFX_DEFAULT_BLE_MOTOR = PFX_CFG_BLE_MOTOR_CONTINUE
PFX_DEFAULT_BLE_ADV_TXPWR = PFX_CFG_BLE_TX_PWR_MAX
PFX_DEFAULT_BLE_SESS_TXPWR = PFX_CFG_BLE_TX_PWR_MAX

# /*******************************************************************************
#  *******************************************************************************
#
#  Event Table Definitions
#
# *******************************************************************************
# *******************************************************************************/
EVT_LUT_MAX = 0x7F
EVT_ID_8879_TWO_BUTTONS = 0x0
EVT_ID_8879_LEFT_BUTTON = 0x1
EVT_ID_8879_RIGHT_BUTTON = 0x2
EVT_ID_8879_LEFT_INC = 0x3
EVT_ID_8879_LEFT_DEC = 0x4
EVT_ID_8879_RIGHT_INC = 0x5
EVT_ID_8879_RIGHT_DEC = 0x6
EVT_ID_8885_LEFT_FWD = 0x7
EVT_ID_8885_LEFT_REV = 0x8
EVT_ID_8885_RIGHT_FWD = 0x9
EVT_ID_8885_RIGHT_REV = 0xA
EVT_ID_8885_LEFT_CTROFF = 0xB
EVT_ID_8885_RIGHT_CTROFF = 0xC
EVT_ID_EV3_BEACON = 0xD
EVT_ID_TEST_EVENT = 0xE
EVT_ID_STARTUP_EVENT = 0xF
EVT_ID_STARTUP_EVENT2 = 0x10
EVT_ID_RCTRAIN_UP = 0x14
EVT_ID_RCTRAIN_DOWN = 0x15
EVT_ID_RCTRAIN_STOP = 0x16
EVT_ID_RCTRAIN_HORN = 0x17
EVT_ID_MAX = 0x20

# /*******************************************************************************
#  Convenient left-shifted versions of the event ID
#    The event ID occupies byte positions <5:2>
#    IR channel occupies byte positions <1:0>
#    These event ID definitions can be logically OR-ed with the channel number
#    for indexing into the event LUT.
# *******************************************************************************/
EVT_8879_TWO_BUTTONS = 0x00
EVT_8879_LEFT_BUTTON = 0x04
EVT_8879_RIGHT_BUTTON = 0x08
EVT_8879_LEFT_INC = 0x0C
EVT_8879_LEFT_DEC = 0x10
EVT_8879_RIGHT_INC = 0x14
EVT_8879_RIGHT_DEC = 0x18
EVT_8885_LEFT_FWD = 0x1C
EVT_8885_LEFT_REV = 0x20
EVT_8885_RIGHT_FWD = 0x24
EVT_8885_RIGHT_REV = 0x28
EVT_8885_LEFT_CTROFF = 0x2C
EVT_8885_RIGHT_CTROFF = 0x30
EVT_EV3_BEACON = 0x34
EVT_TEST_EVENT = 0x38
EVT_DEFAULT_EVENT = 0x3C
EVT_STARTUP_EVENT1 = 0x3C
EVT_STARTUP_EVENT2 = 0x3D
EVT_STARTUP_EVENT3 = 0x3E
EVT_STARTUP_EVENT4 = 0x3F
EVT_STARTUP_EVENT5 = 0x40
EVT_STARTUP_EVENT6 = 0x41
EVT_STARTUP_EVENT7 = 0x42
EVT_STARTUP_EVENT8 = 0x43
EVT_BUTTON_PRESS = 0x44
EVT_BUTTON_LONGPRESS = 0x45
EVT_BUTTON_DOWN = 0x46
EVT_BUTTON_UP = 0x47
EVT_BLE_CONNECT = 0x48
EVT_BLE_DISCONNECT = 0x49
EVT_USB_CONNECT = 0x4A
EVT_USB_DISCONNECT = 0x4B
EVT_RCTRAIN_UP = 0x50
EVT_RCTRAIN_DOWN = 0x54
EVT_RCTRAIN_STOP = 0x58
EVT_RCTRAIN_HORN = 0x5C
EVT_SPARKFUN_POWER = 0x60
EVT_SPARKFUN_A = 0x61
EVT_SPARKFUN_B = 0x62
EVT_SPARKFUN_C = 0x63
EVT_SPARKFUN_UP = 0x64
EVT_SPARKFUN_DOWN = 0x65
EVT_SPARKFUN_LEFT = 0x66
EVT_SPARKFUN_RIGHT = 0x67
EVT_ADAFRUIT_VOLDOWN = 0x68
EVT_ADAFRUIT_PLAY = 0x69
EVT_ADAFRUIT_VOLUP = 0x6A
EVT_ADAFRUIT_SETUP = 0x6B
EVT_ADAFRUIT_STOP = 0x6C
EVT_ADAFRUIT_UP = 0x6D
EVT_ADAFRUIT_DOWN = 0x6E
EVT_ADAFRUIT_LEFT = 0x6F
EVT_ADAFRUIT_RIGHT = 0x70
EVT_ADAFRUIT_ENTER = 0x71
EVT_ADAFRUIT_REPEAT = 0x72
EVT_ADAFRUIT_0 = 0x73
EVT_ADAFRUIT_1 = 0x74
EVT_ADAFRUIT_2 = 0x75
EVT_ADAFRUIT_3 = 0x76
EVT_ADAFRUIT_4 = 0x77
EVT_ADAFRUIT_5 = 0x78
EVT_ADAFRUIT_6 = 0x79
EVT_ADAFRUIT_7 = 0x7A
EVT_ADAFRUIT_8 = 0x7B
EVT_ADAFRUIT_9 = 0x7C
EVT_INVALID = 0xFF
EVT_EVENT_ID_MAX = 0x20
EVT_EVENT_ID_MASK = 0x7C
EVT_EVENT_CH_MASK = 0x03

# /*******************************************************************************
#  Byte indexes
# *******************************************************************************/
EVT_ACT_COMMAND = 0x0
EVT_ACT_MOTOR_ACTION_ID = 0x1
EVT_ACT_MOTOR_MASK = 0x1
EVT_ACT_MOTOR_PARAM1 = 0x2
EVT_ACT_MOTOR_PARAM2 = 0x3
EVT_ACT_LIGHT_FX_ID = 0x4
EVT_ACT_LIGHT_OUTPUT_MASK = 0x5
EVT_ACT_LIGHT_PF_OUTPUT_MASK = 0x6
EVT_ACT_LIGHT_PARAM1 = 0x7
EVT_ACT_LIGHT_PARAM2 = 0x8
EVT_ACT_LIGHT_PARAM3 = 0x9
EVT_ACT_LIGHT_PARAM4 = 0xA
EVT_ACT_LIGHT_PARAM5 = 0xB
EVT_ACT_SOUND_FX_ID = 0xC
EVT_ACT_SOUND_FILE_ID = 0xD
EVT_ACT_SOUND_PARAM1 = 0xE
EVT_ACT_SOUND_PARAM2 = 0xF

# /*******************************************************************************
#  Byte 0 - COMMAND
# *******************************************************************************/
EVT_COMMAND_NONE = 0x00
EVT_COMMAND_ALL_OFF = 0x01
EVT_COMMAND_IR_LOCKOUT_ON = 0x02
EVT_COMMAND_IR_LOCKOUT_OFF = 0x03
EVT_COMMAND_IR_LOCK_TOGGLE = 0x04
EVT_COMMAND_ALL_MOTORS_OFF = 0x05
EVT_COMMAND_ALL_LIGHTS_OFF = 0x06
EVT_COMMAND_ALL_AUDIO_OFF = 0x07
EVT_COMMAND_RESTART = 0x08
EVT_COMMAND_RUN_SCRIPT = 0x09

# /*******************************************************************************
#  Byte 1 - MOTOR_ACTION_ID / MOTOR_MASK
# *******************************************************************************/
EVT_MOTOR_ACTION_ID_MASK = 0xF0
EVT_MOTOR_OUTPUT_MASK = 0x0F
EVT_MOTOR_ESTOP = 0x00
EVT_MOTOR_STOP = 0x10
EVT_MOTOR_INC_SPD = 0x20
EVT_MOTOR_DEC_SPD = 0x30
EVT_MOTOR_INC_SPD_BI = 0x40
EVT_MOTOR_DEC_SPD_BI = 0x50
EVT_MOTOR_CHANGE_DIR = 0x60
EVT_MOTOR_SET_SPD = 0x70
EVT_MOTOR_SET_SPD_TIMED = 0x80
EVT_MOTOR_OSCILLATE = 0x90
EVT_MOTOR_OSCILLATE_BIDIR = 0xA0
EVT_MOTOR_OSCILLATE_BIDIR_WAIT = 0xB0
EVT_MOTOR_WIPE = 0x90
EVT_MOTOR_WIPE_BIDIR = 0xA0
EVT_MOTOR_WIPE_BIDIR_WAIT = 0xB0
EVT_MOTOR_RANDOM = 0xC0
EVT_MOTOR_RANDOM_BIDIR = 0xD0
EVT_MOTOR_SOUND_MODULATED = 0xE0
EVT_MOTOR_SET_SERVO = 0xF0
EVT_MOTOR_OUTPUT_A = 0x01
EVT_MOTOR_OUTPUT_B = 0x02
EVT_MOTOR_OUTPUT_C = 0x04
EVT_MOTOR_OUTPUT_D = 0x08

# /*******************************************************************************
#  Byte 2 - MOTOR_PARAM1
#  Byte 3 - MOTOR_PARAM2
# *******************************************************************************/
EVT_MOTOR_STEP_DEFAULT = 0x0
EVT_MOTOR_STEP_1PCT = 0x1
EVT_MOTOR_STEP_2PCT = 0x2
EVT_MOTOR_STEP_3PCT = 0x3
EVT_MOTOR_STEP_5PCT = 0x4
EVT_MOTOR_STEP_6PCT = 0x5
EVT_MOTOR_STEP_10PCT = 0x6
EVT_MOTOR_STEP_20PCT = 0x7
EVT_MOTOR_STEP_25PCT = 0x8
EVT_MOTOR_STEP_33PCT = 0x9
EVT_MOTOR_STEP_TLG7STEP = 0xA
EVT_MOTOR_STEP_SERVO = 0xB
EVT_MOTOR_SPEED_STOP = 0x0
EVT_MOTOR_SPEED_FWD_10 = 0x1
EVT_MOTOR_SPEED_FWD_25 = 0x2
EVT_MOTOR_SPEED_FWD_33 = 0x3
EVT_MOTOR_SPEED_FWD_50 = 0x4
EVT_MOTOR_SPEED_FWD_67 = 0x5
EVT_MOTOR_SPEED_FWD_75 = 0x6
EVT_MOTOR_SPEED_FWD_100 = 0x7
EVT_MOTOR_SPEED_REV_STOP = 0x8
EVT_MOTOR_SPEED_REV_10 = 0x9
EVT_MOTOR_SPEED_REV_25 = 0xA
EVT_MOTOR_SPEED_REV_33 = 0xB
EVT_MOTOR_SPEED_REV_50 = 0xC
EVT_MOTOR_SPEED_REV_67 = 0xD
EVT_MOTOR_SPEED_REV_75 = 0xE
EVT_MOTOR_SPEED_REV_100 = 0xF
EVT_MOTOR_SPEED_LO_MASK = 0xF
EVT_MOTOR_SPEED_HIRES_MASK = 0x3F
EVT_MOTOR_SPEED_HIRES = 0x80
EVT_MOTOR_SPEED_HIRES_REV = 0x40
EVT_MOTOR_SPEED_HIRES_FWD = 0x80
EVT_MOTOR_PERIOD_250MS = 0x0
EVT_MOTOR_PERIOD_500MS = 0x1
EVT_MOTOR_PERIOD_750MS = 0x2
EVT_MOTOR_PERIOD_1S = 0x3
EVT_MOTOR_PERIOD_1_25S = 0x4
EVT_MOTOR_PERIOD_1_5S = 0x5
EVT_MOTOR_PERIOD_2S = 0x6
EVT_MOTOR_PERIOD_2_5S = 0x7
EVT_MOTOR_PERIOD_3S = 0x8
EVT_MOTOR_PERIOD_4S = 0x9
EVT_MOTOR_PERIOD_5S = 0xA
EVT_MOTOR_PERIOD_10S = 0xB
EVT_MOTOR_PERIOD_15S = 0xC
EVT_MOTOR_PERIOD_20S = 0xD
EVT_MOTOR_PERIOD_30S = 0xE
EVT_MOTOR_PERIOD_60S = 0xF
EVT_MOTOR_PERIOD_ON_MASK = 0x0F
EVT_MOTOR_PERIOD_OFF_MASK = 0xF0
EVT_MOTOR_POS_NEG90 = 0x0
EVT_MOTOR_POS_NEG75 = 0x1
EVT_MOTOR_POS_NEG60 = 0x2
EVT_MOTOR_POS_NEG45 = 0x3
EVT_MOTOR_POS_NEG30 = 0x4
EVT_MOTOR_POS_NEG15 = 0x5
EVT_MOTOR_POS_0 = 0x6
EVT_MOTOR_POS_POS15 = 0x7
EVT_MOTOR_POS_POS30 = 0x8
EVT_MOTOR_POS_POS45 = 0x9
EVT_MOTOR_POS_POS60 = 0xA
EVT_MOTOR_POS_POS75 = 0xB
EVT_MOTOR_POS_POS90 = 0xC

# /*******************************************************************************
#  Byte 4 - LIGHT_FX_ID
# *******************************************************************************/
EVT_LIGHT_COMBO_MASK = 0x80
EVT_LIGHT_ID_MASK = 0x7F
EVT_LIGHTFX_NONE = 0x00
EVT_LIGHTFX_ON_OFF_TOGGLE = 0x01
EVT_LIGHTFX_INC_BRIGHT = 0x02
EVT_LIGHTFX_DEC_BRIGHT = 0x03
EVT_LIGHTFX_SET_BRIGHT = 0x04
EVT_LIGHTFX_FLASH50_P = 0x05
EVT_LIGHTFX_FLASH50_N = 0x06
EVT_LIGHTFX_STROBE_P = 0x07
EVT_LIGHTFX_STROBE_N = 0x08
EVT_LIGHTFX_GYRALITE_P = 0x09
EVT_LIGHTFX_GYRALITE_N = 0x0A
EVT_LIGHTFX_FLICKER = 0x0B
EVT_LIGHTFX_RAND_BLINK = 0x0C
EVT_LIGHTFX_PHOTON_TORP = 0x0D
EVT_LIGHTFX_LASER_PULSE = 0x0E
EVT_LIGHTFX_ENGINE_GLOW = 0x0F
EVT_LIGHTFX_LIGHTHOUSE = 0x10
EVT_LIGHTFX_BROKEN_LIGHT = 0x11
EVT_LIGHTFX_STATUS_IND = 0x12
EVT_LIGHTFX_SOUND_MOD = 0x13
EVT_LIGHTFX_MOTOR_MOD = 0x14
EVT_COMBOFX_NONE = 0x00
EVT_COMBOFX_LIN_SWEEP = 0x01
EVT_COMBOFX_BARGRAPH = 0x02
EVT_COMBOFX_KNIGHTRIDER = 0x03
EVT_COMBOFX_EMCY_TWSONIC = 0x04
EVT_COMBOFX_EMCY_WHELEN = 0x05
EVT_COMBOFX_TIMES_SQ = 0x06
EVT_COMBOFX_NOISE = 0x07
EVT_COMBOFX_TWINKLE_STAR = 0x08
EVT_COMBOFX_TRAFFIC_SIG = 0x09
EVT_COMBOFX_SOUND_BAR = 0x0A
EVT_COMBOFX_ALT_FLASH = 0x0B
EVT_COMBOFX_LAVA_LAMP = 0x0C
EVT_COMBOFX_LASER_CANNON = 0x0D
EVT_COMBOFX_DRAGSTER = 0x0E
EVT_COMBOFX_RUNWAY = 0x0F
EVT_COMBOFX_FORMULA1 = 0x10

# /*******************************************************************************
#  Byte 5 - LIGHT_OUTPUT_MASK
# *******************************************************************************/
EVT_LIGHT_OUTPUT_1 = 0x01
EVT_LIGHT_OUTPUT_2 = 0x02
EVT_LIGHT_OUTPUT_3 = 0x04
EVT_LIGHT_OUTPUT_4 = 0x08
EVT_LIGHT_OUTPUT_5 = 0x10
EVT_LIGHT_OUTPUT_6 = 0x20
EVT_LIGHT_OUTPUT_7 = 0x40
EVT_LIGHT_OUTPUT_8 = 0x80

# /*******************************************************************************
#  Byte 6 - LIGHT_PF_OUTPUT_MASK
# *******************************************************************************/
EVT_LIGHT_PF_OUTPUT_A = 0x01
EVT_LIGHT_PF_OUTPUT_B = 0x02
EVT_LIGHT_PF_OUTPUT_C = 0x04
EVT_LIGHT_PF_OUTPUT_D = 0x08

# /*******************************************************************************
#  Byte  7 - LIGHT_PARAM1
#  Byte  8 - LIGHT_PARAM2
#  Byte  9 - LIGHT_PARAM3
#  Byte 10 - LIGHT_PARAM4
#  Byte 11 - LIGHT_PARAM5
# *******************************************************************************/
EVT_DIR_OPTION_NONE = 0x00
EVT_DIR_OPTION_MOTORA_FWD = 0x01
EVT_DIR_OPTION_MOTORA_REV = 0x02
EVT_DIR_OPTION_MOTORB_FWD = 0x03
EVT_DIR_OPTION_MOTORB_REV = 0x04
EVT_DIR_OPTION_MOTORC_FWD = 0x05
EVT_DIR_OPTION_MOTORC_REV = 0x06
EVT_DIR_OPTION_MOTORD_FWD = 0x07
EVT_DIR_OPTION_MOTORD_REV = 0x08
EVT_DIR_OPTION_ODD_MOTORA_FWD = 0x09
EVT_DIR_OPTION_ODD_MOTORB_FWD = 0x0A
EVT_DIR_OPTION_ODD_MOTORC_FWD = 0x0B
EVT_DIR_OPTION_ODD_MOTORD_FWD = 0x0C
EVT_DIR_OPTION_ODD_MOTORA_REV = 0x0D
EVT_DIR_OPTION_ODD_MOTORB_REV = 0x0E
EVT_DIR_OPTION_ODD_MOTORC_REV = 0x0F
EVT_DIR_OPTION_ODD_MOTORD_REV = 0x10
EVT_FADE_TIME_NONE = 0x0
EVT_FADE_TIME_50MS = 0x1
EVT_FADE_TIME_100MS = 0x2
EVT_FADE_TIME_200MS = 0x3
EVT_FADE_TIME_400MS = 0x4
EVT_FADE_TIME_500MS = 0x5
EVT_FADE_TIME_600MS = 0x6
EVT_FADE_TIME_800MS = 0x7
EVT_FADE_TIME_1S = 0x8
EVT_FADE_TIME_1_5S = 0x9
EVT_FADE_TIME_2S = 0xA
EVT_FADE_TIME_2_5S = 0xB
EVT_FADE_TIME_3S = 0xC
EVT_FADE_TIME_4S = 0xD
EVT_FADE_TIME_5S = 0xE
EVT_FADE_TIME_10S = 0xF
EVT_FADE_TIME_MAX = 0x10
EVT_FADE_TIME_MIN = 0x1F
EVT_FADE_FACTOR_NONE = 0x0
EVT_FADE_FACTOR_1 = 0x1
EVT_FADE_FACTOR_5 = 0x2
EVT_FADE_FACTOR_10 = 0x3
EVT_FADE_FACTOR_15 = 0x4
EVT_FADE_FACTOR_20 = 0x5
EVT_FADE_FACTOR_25 = 0x6
EVT_FADE_FACTOR_30 = 0x7
EVT_FADE_FACTOR_40 = 0x8
EVT_FADE_FACTOR_50 = 0x9
EVT_FADE_FACTOR_75 = 0xA
EVT_FADE_FACTOR_90 = 0xB
EVT_FADE_FACTOR_100 = 0xC
EVT_FADE_FACTOR_150 = 0xD
EVT_FADE_FACTOR_200 = 0xE
EVT_FADE_FACTOR_400 = 0xF
EVT_FADE_FACTOR_MAX = 0x10
EVT_FADE_FACTOR_MIN = 0x1F
EVT_PERIOD_100MS = 0x0
EVT_PERIOD_250MS = 0x1
EVT_PERIOD_500MS = 0x2
EVT_PERIOD_750MS = 0x3
EVT_PERIOD_1S = 0x4
EVT_PERIOD_1_25S = 0x5
EVT_PERIOD_1_5S = 0x6
EVT_PERIOD_1_75S = 0x7
EVT_PERIOD_2S = 0x8
EVT_PERIOD_2_5S = 0x9
EVT_PERIOD_3S = 0xA
EVT_PERIOD_4S = 0xB
EVT_PERIOD_5S = 0xC
EVT_PERIOD_8S = 0xD
EVT_PERIOD_10S = 0xE
EVT_PERIOD_20S = 0xF
EVT_PERIOD_MAX = 0x10
EVT_PERIOD2_50MS = 0x0
EVT_PERIOD2_100MS = 0x1
EVT_PERIOD2_200MS = 0x2
EVT_PERIOD2_300MS = 0x3
EVT_PERIOD2_400MS = 0x4
EVT_PERIOD2_500MS = 0x5
EVT_PERIOD2_600MS = 0x6
EVT_PERIOD2_700MS = 0x7
EVT_PERIOD2_800MS = 0x8
EVT_PERIOD2_900MS = 0x9
EVT_PERIOD2_1S = 0xA
EVT_PERIOD2_1_25S = 0xB
EVT_PERIOD2_1_5S = 0xC
EVT_PERIOD2_1_75S = 0xD
EVT_PERIOD2_2S = 0xE
EVT_PERIOD2_3S = 0xF
EVT_PERIOD2_MAX = 0x10
PFX_PERIOD_COUNT_MAX = EVT_PERIOD_MAX + EVT_PERIOD2_MAX
EVT_DUTYCY_1 = 0x0
EVT_DUTYCY_2 = 0x1
EVT_DUTYCY_5 = 0x2
EVT_DUTYCY_10 = 0x3
EVT_DUTYCY_15 = 0x4
EVT_DUTYCY_20 = 0x5
EVT_DUTYCY_25 = 0x6
EVT_DUTYCY_30 = 0x7
EVT_DUTYCY_40 = 0x8
EVT_DUTYCY_50 = 0x9
EVT_DUTYCY_60 = 0xA
EVT_DUTYCY_70 = 0xB
EVT_DUTYCY_75 = 0xC
EVT_DUTYCY_80 = 0xD
EVT_DUTYCY_85 = 0xE
EVT_DUTYCY_90 = 0xF
EVT_DUTYCY_95 = 0x10
EVT_DUTYCY_98 = 0x11
EVT_DUTYCY_99 = 0x12
EVT_DUTYCY_MAX = 0x13
EVT_BURST_COUNT_1 = 0x0
EVT_BURST_COUNT_2 = 0x1
EVT_BURST_COUNT_3 = 0x2
EVT_BURST_COUNT_4 = 0x3
EVT_BURST_COUNT_MAX = 0x4
EVT_SIZE_8_LIGHTS = 0x0
EVT_SIZE_7_LIGHTS = 0x1
EVT_SIZE_6_LIGHTS = 0x2
EVT_SIZE_5_LIGHTS = 0x3
EVT_SIZE_4_LIGHTS = 0x4
EVT_BAR_STYLE_NONE = 0x0
EVT_BAR_STYLE_LEFTRIGHT = 0x1
EVT_BAR_STYLE_RIGHTLEFT = 0x2
EVT_BAR_STYLE_INOUT = 0x3
EVT_BAR_STYLE_OUTIN = 0x4
EVT_SWEEP_STYLE_LEFTRIGHT = 0x0
EVT_SWEEP_STYLE_RIGHTLEFT = 0x1
EVT_TWINSONIC_SINGLE = 0x0
EVT_TWINSONIC_DUAL = 0x1
EVT_TWINSONIC_AERO = 0x2
EVT_TWINSONIC_COMBO = 0x3
EVT_WHELEN_SIGALERT = 0x0
EVT_WHELEN_SIGALERT_STDY = 0x1
EVT_WHELEN_COMET_FLASH = 0x2
EVT_WHELEN_ACT_FLASH50 = 0x3
EVT_WHELEN_ACT_FLASH150 = 0x4
EVT_WHELEN_MODU_FLASH = 0x5
EVT_WHELEN_SINGLE_FLASH = 0x6
EVT_WHELEN_DOUBLE_FLASH = 0x7
EVT_WHELEN_TRIPLE_FLASH = 0x8
EVT_WHELEN_WARNING = 0x9
EVT_WHELEN_RANDOM = 0xA
EVT_SEQ_SOLID = 0x0
EVT_SEQ_LEFTRIGHT = 0x1
EVT_SEQ_INOUT = 0x2
EVT_FLASH_RATE_SLOW = 0x0
EVT_FLASH_RATE_MED = 0x1
EVT_FLASH_RATE_FAST = 0x2
EVT_FLASH_RATE_VERYFAST = 0x3
EVT_TRAFFIC_STD = 0x0
EVT_TRAFFIC_STD_FLGRN = 0x1
EVT_TRAFFIC_EUR = 0x2
EVT_TRAFFIC_NS_FLRED = 0x3
EVT_TRAFFIC_STD_PED = 0x4
EVT_TRAFFIC_STD_FLGRN_PED = 0x5
EVT_TRAFFIC_EUR_PED = 0x6
EVT_TRAFFIC_EW_FLRED = 0x7
EVT_TRAFFIC_EUR2 = 0x8
EVT_TRAFFIC_EUR2_PED = 0x9
EVT_TRAFFIC_INT = 0x8
EVT_TRAFFIC_INT_PED = 0x9
EVT_TRAFFIC_INT2 = 0xA
EVT_TRAFFIC_INT2_PED = 0xB
EVT_SEQ_TIME_SLOW = 0x0
EVT_SEQ_TIME_MED = 0x1
EVT_SEQ_TIME_FAST = 0x2
EVT_SEQ_TIME_VERYFAST = 0x3
EVT_FAULT_RATE_RARE = 0x0
EVT_FAULT_RATE_OCCASIONAL = 0x1
EVT_FAULT_RATE_OFTEN = 0x2
EVT_FAULT_RATE_VERYOFTEN = 0x3
EVT_FAULT_RATE_ALWAYS = 0x4
EVT_FAULT_INT_SUBTLE = 0x0
EVT_FAULT_INT_MODERATE = 0x1
EVT_FAULT_INT_SERVERE = 0x2
EVT_FAULT_INT_MAX = 0x3
EVT_FAULT_INT_FLICKER_ON = 0x4
EVT_SOURCE1_USB_CONN = 0x01
EVT_SOURCE1_USB_ACTIVITY = 0x02
EVT_SOURCE1_IR_ACTIVITY = 0x04
EVT_SOURCE1_IR_LOCKOUT = 0x08
EVT_SOURCE1_AUDIO_PLAY = 0x10
EVT_SOURCE1_BLE_CONN = 0x20
EVT_SOURCE1_BLE_ACTIVITY = 0x40
EVT_SOURCE1_FS_ACCESS = 0x80
EVT_SOURCE2_MOTORA_FWD = 0x01
EVT_SOURCE2_MOTORA_REV = 0x02
EVT_SOURCE2_MOTORB_FWD = 0x04
EVT_SOURCE2_MOTORB_REV = 0x08
EVT_SOURCE2_MOTORC_FWD = 0x10
EVT_SOURCE2_MOTORC_REV = 0x20
EVT_SOURCE2_MOTORD_FWD = 0x40
EVT_SOURCE2_MOTORD_REV = 0x80
EVT_SOURCE2_BUTTON = 0x40
EVT_SOURCE2_MOTORA_MASK = 0xFC
EVT_SOURCE2_MOTORB_MASK = 0xF3
EVT_SOURCE2_MOTORC_MASK = 0xCF
EVT_SOURCE2_MOTORD_MASK = 0x3F
EVT_INVERT_NORMAL = 0x00
EVT_TRANSITION_TOGGLE = 0x00
EVT_TRANSITION_ON = 0x01
EVT_TRANSITION_OFF = 0x02
EVT_TRANSITION_DURATION = 0x03
EVT_TRANSITION_MASK = 0x03
EVT_DRAGSTER_STD = 0x00
EVT_DRAGSTER_PRO_05 = 0x01
EVT_DRAGSTER_PRO_04 = 0x02
EVT_F1_RACE_START = 0x00
EVT_F1_COUNTDOWN = 0x01
EVT_F1_RACE_BREAK = 0x02
EVT_F1_TRAINING_START = 0x03
EVT_F1_TRAINING_BREAK = 0x04
EVT_F1_TRAINING_END = 0x05
EVT_RUNWAY_RATE_STEADY = 0x00
EVT_RUNWAY_RATE_SLOW = 0x00
EVT_RUNWAY_RATE_MED = 0x00
EVT_RUNWAY_RATE_FAST = 0x00
EVT_RUNWAY_BRIGHT_MAX = 0x00
EVT_RUNWAY_BRIGHT_MED = 0x01
EVT_RUNWAY_BRIGHT_LOW = 0x02
EVT_RUNWAY_BRIGHT_MIN = 0x03

# /*******************************************************************************
#  Byte 12 - SOUND_FX_ID
# *******************************************************************************/
EVT_SOUND_NONE = 0x0
EVT_SOUND_INC_VOL = 0x1
EVT_SOUND_DEC_VOL = 0x2
EVT_SOUND_SET_VOL = 0x3
EVT_SOUND_PLAY_ONCE = 0x4
EVT_SOUND_PLAY_CONT = 0x5
EVT_SOUND_PLAY_NTIMES = 0x6
EVT_SOUND_PLAY_DUR = 0x7
EVT_SOUND_PLAY_PITCH = 0x8
EVT_SOUND_PLAY_GATED = 0x9
EVT_SOUND_PLAY_AM = 0xA
EVT_SOUND_STOP = 0xB
EVT_SOUND_PLAY_IDX_MOTOR = 0xC
EVT_SOUND_PLAY_RAND = 0xD
EVT_SOUND_PLAY_PAUSE_FLAG = 0x10

# /*******************************************************************************
#  Byte 14 - SOUND_PARAM1
#  Byte 15 - SOUND_PARAM2
# *******************************************************************************/
EVT_SOUND_DUR_500MS = 0x0
EVT_SOUND_DUR_1S = 0x1
EVT_SOUND_DUR_1_5S = 0x2
EVT_SOUND_DUR_2S = 0x3
EVT_SOUND_DUR_3S = 0x4
EVT_SOUND_DUR_4S = 0x5
EVT_SOUND_DUR_5S = 0x6
EVT_SOUND_DUR_10S = 0x7
EVT_SOUND_DUR_15S = 0x8
EVT_SOUND_DUR_20S = 0x9
EVT_SOUND_DUR_30S = 0xA
EVT_SOUND_DUR_45S = 0xB
EVT_SOUND_DUR_60S = 0xC
EVT_SOUND_DUR_90S = 0xD
EVT_SOUND_DUR_2M = 0xE
EVT_SOUND_DUR_5M = 0xF
EVT_SOUND_TOGGLE = 0x0
EVT_SOUND_RESTART = 0x1
EVT_SOUND_IDX_MOTOR_MASK = 0x03
EVT_SOUND_IDX_MOTOR_CURR_SPD = 0x04
EVT_SOUND_IDX_MOTOR_TGT_SPD = 0x00
EVT_SOUND_IDX_VOL_MOD_OFF = 0x0
EVT_SOUND_IDX_VOL_MOD_LIGHT = 0x1
EVT_SOUND_IDX_VOL_MOD_MED = 0x2
EVT_SOUND_IDX_VOL_MOD_HEAVY = 0x3
EVT_SOUND_IDX_VOL_MOD_MASK = 0x3
EVT_SOUND_IDX_PLAY_STARTUP = 0x4
EVT_SOUND_IDX_PLAY_START_OVR = 0x8
EVT_SOUND_RAND_RARE = 0x0
EVT_SOUND_RAND_OCCASIONAL = 0x1
EVT_SOUND_RAND_OFTEN = 0x2
EVT_SOUND_RAND_VERYOFTEN = 0x3
PFX_SOUND_IDX_MAX_NOTCHES = 8

PFX_SOUND_IDX_STARTUP = 0xEF
PFX_SOUND_IDX_SHUTDOWN = 0xF7
PFX_SOUND_IDX_NOTCH1_LOOP = 0xE0
PFX_SOUND_IDX_NOTCH2_LOOP = 0xE1
PFX_SOUND_IDX_NOTCH3_LOOP = 0xE2
PFX_SOUND_IDX_NOTCH4_LOOP = 0xE3
PFX_SOUND_IDX_NOTCH5_LOOP = 0xE4
PFX_SOUND_IDX_NOTCH6_LOOP = 0xE5
PFX_SOUND_IDX_NOTCH7_LOOP = 0xE6
PFX_SOUND_IDX_NOTCH8_LOOP = 0xE7
PFX_SOUND_IDX_ACCEL1 = 0xE8
PFX_SOUND_IDX_ACCEL2 = 0xE9
PFX_SOUND_IDX_ACCEL3 = 0xEA
PFX_SOUND_IDX_ACCEL4 = 0xEB
PFX_SOUND_IDX_ACCEL5 = 0xEC
PFX_SOUND_IDX_ACCEL6 = 0xED
PFX_SOUND_IDX_ACCEL7 = 0xEE
PFX_SOUND_IDX_DECEL1 = 0xF0
PFX_SOUND_IDX_DECEL2 = 0xF1
PFX_SOUND_IDX_DECEL3 = 0xF2
PFX_SOUND_IDX_DECEL4 = 0xF3
PFX_SOUND_IDX_DECEL5 = 0xF4
PFX_SOUND_IDX_DECEL6 = 0xF5
PFX_SOUND_IDX_DECEL7 = 0xF6
PFX_SOUND_IDX_CHANGE_DIR = 0xFA
PFX_SOUND_IDX_SET_OFF = 0xFB
PFX_SOUND_IDX_RAPID_ACCEL = 0xFC
PFX_SOUND_IDX_RAPID_DECEL = 0xFD
PFX_SOUND_IDX_BRAKE_STOP = 0xFE
PFX_SOUND_IDX_GATED_NOTCH11 = 0xD0
PFX_SOUND_IDX_GATED_NOTCH12 = 0xD1
PFX_SOUND_IDX_GATED_NOTCH13 = 0xD2
PFX_SOUND_IDX_GATED_NOTCH14 = 0xD3
PFX_SOUND_IDX_GATED_NOTCH21 = 0xD4
PFX_SOUND_IDX_GATED_NOTCH22 = 0xD5
PFX_SOUND_IDX_GATED_NOTCH23 = 0xD6
PFX_SOUND_IDX_GATED_NOTCH24 = 0xD7
PFX_SOUND_IDX_GATED_NOTCH31 = 0xD8
PFX_SOUND_IDX_GATED_NOTCH32 = 0xD9
PFX_SOUND_IDX_GATED_NOTCH33 = 0xDA
PFX_SOUND_IDX_GATED_NOTCH34 = 0xDB
PFX_SOUND_IDX_GATED_NOTCH41 = 0xDC
PFX_SOUND_IDX_GATED_NOTCH42 = 0xDD
PFX_SOUND_IDX_GATED_NOTCH43 = 0xDE
PFX_SOUND_IDX_GATED_NOTCH44 = 0xDF

# /*******************************************************************************
#  *******************************************************************************
#
#  Interface Command Byte Definitions
#
# *******************************************************************************
# *******************************************************************************/
PFX_BLE_CMD_PRE_DELIMITER = 0x5B
PFX_BLE_CMD_PRE_DELIMITER_LEN = 3
PFX_BLE_CMD_POST_DELIMITER = 0x5D
PFX_BLE_CMD_POST_DELIMITER_LEN = 3

# /*******************************************************************************
#  Operation and Configuration Commands
# *******************************************************************************/
PFX_USB_CMD_GET_ICD_REV = 0x08
PFX_USB_CMD_GET_STATUS = 0x01
PFX_USB_CMD_SET_FACTORY_DEFAULTS = 0x02
PFX_USB_CMD_GET_CONFIG = 0x03
PFX_USB_CMD_SET_CONFIG = 0x04
PFX_USB_CMD_VERIFY_CONFIG = 0x05
PFX_USB_CMD_GET_CURRENT_STATE = 0x06
PFX_USB_CMD_GET_NAME = 0x07
PFX_USB_CMD_SET_NAME = 0x09
PFX_CMD_GET_ICD_REV = 0x08
PFX_CMD_GET_STATUS = 0x01
PFX_CMD_SET_FACTORY_DEFAULTS = 0x02
PFX_CMD_GET_CONFIG = 0x03
PFX_CMD_SET_CONFIG = 0x04
PFX_CMD_VERIFY_CONFIG = 0x05
PFX_CMD_GET_CURRENT_STATE = 0x06
PFX_CMD_GET_NAME = 0x07
PFX_CMD_SET_NAME = 0x09

# /*******************************************************************************
#  Event/Action LUT Commands
# *******************************************************************************/
PFX_USB_CMD_VERIFY_EVENT_LUT = 0x10
PFX_USB_CMD_GET_EVENT_ACTION = 0x11
PFX_USB_CMD_SET_EVENT_ACTION = 0x12
PFX_USB_CMD_TEST_ACTION = 0x13
PFX_USB_CMD_GET_LAST_IR_MSG = 0x14
PFX_CMD_VERIFY_EVENT_LUT = 0x10
PFX_CMD_GET_EVENT_ACTION = 0x11
PFX_CMD_SET_EVENT_ACTION = 0x12
PFX_CMD_TEST_ACTION = 0x13
PFX_CMD_GET_LAST_IR_MSG = 0x14
PFX_CMD_SEND_EVENT = 0x15

# /*******************************************************************************
#  Audio Commands
# *******************************************************************************/
PFX_USB_CMD_INC_VOLUME = 0x20
PFX_USB_CMD_DEC_VOLUME = 0x21
PFX_USB_CMD_GET_AUDIO_LUT_ENTRY = 0x22
PFX_USB_CMD_GET_AUDIO_CAPACITY = 0x23
PFX_USB_CMD_ERASE_AUDIO_LUT = 0x24
PFX_USB_CMD_ADD_AUDIO_FILE = 0x25
PFX_USB_CMD_ADD_AUDIO_DATA = 0x26
PFX_USB_CMD_ADD_AUDIO_DONE = 0x27
PFX_USB_CMD_GET_AUDIO_FILE = 0x28
PFX_USB_CMD_GET_AUDIO_DATA = 0x29
PFX_USB_CMD_SET_AUDIO_EQ = 0x2A
PFX_CMD_INC_VOLUME = 0x20
PFX_CMD_DEC_VOLUME = 0x21
PFX_CMD_SET_AUDIO_EQ = 0x2A

# /*******************************************************************************
#  File Commands
# *******************************************************************************/
PFX_USB_CMD_FILE_OPEN = 0x40
PFX_USB_CMD_FILE_CLOSE = 0x41
PFX_USB_CMD_FILE_READ = 0x42
PFX_USB_CMD_FILE_WRITE = 0x43
PFX_USB_CMD_FILE_SEEK = 0x44
PFX_USB_CMD_FILE_DIR = 0x45
PFX_USB_CMD_FILE_REMOVE = 0x46
PFX_USB_CMD_FILE_FORMAT_FS = 0x47
PFX_USB_CMD_FILE_GET_FS_STATE = 0x48
PFX_USB_CMD_GET_FLASH_SECTORMAP = 0x49
PFX_USB_CMD_GET_FLASH_DIR_ENTRY = 0x4A
PFX_CMD_FILE_OPEN = 0x40
PFX_CMD_FILE_CLOSE = 0x41
PFX_CMD_FILE_READ = 0x42
PFX_CMD_FILE_WRITE = 0x43
PFX_CMD_FILE_SEEK = 0x44
PFX_CMD_FILE_DIR = 0x45
PFX_CMD_FILE_REMOVE = 0x46
PFX_CMD_FILE_FORMAT_FS = 0x47
PFX_CMD_FILE_GET_FS_STATE = 0x48
PFX_CMD_GET_FLASH_SECTORMAP = 0x49
PFX_CMD_GET_FLASH_DIR_ENTRY = 0x4A
PFX_CMD_RUN_SCRIPT = 0x4B
PFX_CMD_FILE_WRITE_FAST = 0x4C
PFX_CMD_FILE_READ_FAST = 0x4D
PFX_MSG_FILE_READ_FAST = 0xCD

# /*******************************************************************************
#  Bluetooth Interface Commands
# *******************************************************************************/
PFX_USB_CMD_GET_BT_STATUS = 0x50
PFX_USB_CMD_SET_BT_POWER = 0x51
PFX_USB_CMD_SEND_BT_UART = 0x52
PFX_USB_CMD_RECEIVE_BT_UART = 0x53
PFX_CMD_GET_BT_STATUS = 0x50
PFX_CMD_SET_BT_POWER = 0x51
PFX_CMD_SEND_BT_UART = 0x52
PFX_CMD_RECEIVE_BT_UART = 0x53

# /*******************************************************************************
#  Notifications
# *******************************************************************************/
PFX_CMD_SET_NOTIFICATIONS = 0x60
PFX_MSG_NOTIFICATION = 0x61

# /*******************************************************************************
#  Service Commands
# *******************************************************************************/
PFX_USB_CMD_LOAD_FIRMWARE_FILE = 0x30
PFX_USB_CMD_LOAD_FIRMWARE_DATA = 0x31
PFX_USB_CMD_LOAD_FIRMWARE_DONE = 0x32
PFX_USB_CMD_READ_NVRAM = 0x33
PFX_USB_CMD_READ_BOOTCONFIG = 0x34
PFX_USB_CMD_ERASE_NVRAM = 0x35
PFX_USB_CMD_COMPUTE_CRC32 = 0x36
PFX_USB_CMD_REBOOT = 0x37
PFX_USB_CMD_WRITE_SN = 0x38
PFX_USB_CMD_READ_SN = 0x39
PFX_CMD_LOAD_FIRMWARE_FILE = 0x30
PFX_CMD_LOAD_FIRMWARE_DATA = 0x31
PFX_CMD_LOAD_FIRMWARE_DONE = 0x32
PFX_CMD_READ_NVRAM = 0x33
PFX_CMD_READ_BOOTCONFIG = 0x34
PFX_CMD_ERASE_NVRAM = 0x35
PFX_CMD_COMPUTE_CRC32 = 0x36
PFX_CMD_REBOOT = 0x37
PFX_CMD_WRITE_SN = 0x38
PFX_CMD_READ_SN = 0x39

# /*******************************************************************************
#  Low Level Test/Debug Commands
# *******************************************************************************/
PFX_USB_CMD_STATUS_LED = 0x70
PFX_USB_CMD_DIAG_LED = 0x71
PFX_USB_CMD_WRITE_SPI = 0x72
PFX_USB_CMD_READ_SPI = 0x73
PFX_USB_CMD_WRITE_I2C = 0x74
PFX_USB_CMD_READ_I2C = 0x75
PFX_USB_CMD_READ_FLASH = 0x76
PFX_USB_CMD_GET_IRRX_STATUS = 0x77
PFX_CMD_STATUS_LED = 0x70
PFX_CMD_WRITE_SPI = 0x72
PFX_CMD_READ_SPI = 0x73
PFX_CMD_WRITE_I2C = 0x74
PFX_CMD_READ_I2C = 0x75
PFX_CMD_READ_FLASH = 0x76
PFX_CMD_GET_IRRX_STATUS = 0x77

# /*******************************************************************************
#  *******************************************************************************
#
#  Interface convenient constant defines and codes
#
# *******************************************************************************
# *******************************************************************************/
PFX_RESET_BYTE0 = 0xAA
PFX_RESET_BYTE1 = 0x55
PFX_RESET_BYTE2 = 0xDE
PFX_RESET_BYTE3 = 0xAD
PFX_RESET_BYTE4 = 0xBE
PFX_RESET_BYTE5 = 0xEF
PFX_RESET_BYTE6 = 0x02
PFX_STATUS_BYTE0 = 0xA5
PFX_STATUS_BYTE1 = 0x5A
PFX_STATUS_BYTE2 = 0x6E
PFX_STATUS_BYTE3 = 0x40
PFX_STATUS_BYTE4 = 0x54
PFX_STATUS_BYTE5 = 0xA4
PFX_STATUS_BYTE6 = 0xE5
PFX_GET_ICD_BYTE0 = 0x60
PFX_GET_ICD_BYTE1 = 0x0D
PFX_GET_ICD_BYTE2 = 0x01
PFX_REBOOT_BYTE0 = 0x5A
PFX_REBOOT_BYTE1 = 0xA5
PFX_REBOOT_BYTE2 = 0xD0
PFX_REBOOT_BYTE3 = 0xBE
PFX_REBOOT_BYTE4 = 0xB0
PFX_REBOOT_BYTE5 = 0x04
PFX_REBOOT_BYTE6 = 0x77
PFX_ERASE_NVRAM_BYTE0 = 0xEE
PFX_ERASE_NVRAM_BYTE1 = 0x4A
PFX_ERASE_NVRAM_BYTE2 = 0x5E
PFX_ERASE_NVRAM_BYTE3 = 0xEE
PFX_ERASE_NVRAM_BYTE4 = 0x4A
PFX_ERASE_NVRAM_BYTE5 = 0x5E
PFX_ERASE_NVRAM_BYTE6 = 0x35
PFX_WRITE_SN_BYTE0 = 0x5E
PFX_WRITE_SN_BYTE1 = 0x45
PFX_WRITE_SN_BYTE2 = 0x5E
PFX_WRITE_SN_BYTE3 = 0x41
PFX_WRITE_SN_BYTE4 = 0xA1
PFX_WRITE_SN_BYTE5 = 0x10
PFX_WRITE_SN_BYTE6 = 0x70
PFX_FORMAT_BYTE0 = 0xEA
PFX_FORMAT_BYTE1 = 0x5E
PFX_FORMAT_BYTE2 = 0x88
PFX_FILE_ACC_READ = 0x01
PFX_FILE_ACC_WRITE = 0x02
PFX_FILE_ACC_CREATE = 0x04
PFX_DIR_REQ_GET_FILE_COUNT = 0x00
PFX_DIR_REQ_GET_FREE_SPACE = 0x01
PFX_DIR_REQ_GET_DIR_ENTRY_IDX = 0x02
PFX_DIR_REQ_GET_DIR_ENTRY_ID = 0x03
PFX_DIR_REQ_ADD_AUDIO_FILE_ID = 0x04
PFX_DIR_REQ_RENAME_FILE_ID = 0x05
PFX_DIR_REQ_SET_ATTR_ID = 0x06
PFX_DIR_REQ_SET_USER_DATA1_ID = 0x07
PFX_DIR_REQ_SET_USER_DATA2_ID = 0x08
PFX_DIR_REQ_COMPUTE_CRC32_ID = 0x09
PFX_DIR_REQ_SET_ATTR_MASKED_ID = 0x0A
PFX_DIR_REQ_GET_NAMED_FILE_ID = 0x0B
PFX_DIR_REQ_GET_SMALL_DIR_ID = 0x0C
PFX_DIR_REQ_GET_SMALL_DIR_IDX = 0x0D
PFX_FILE_FMT_MASK = 0xFF00
PFX_FILE_FMT_WAV = 0x0000
PFX_FILE_FMT_FLAC = 0x0100
PFX_FILE_FMT_MP3 = 0x0200
PFX_FILE_FMT_OGG = 0x0300
PFX_FILE_FMT_AU = 0x0400
PFX_FILE_FMT_GSM = 0x0500
PFX_FILE_FMT_TXT = 0x1000
PFX_FILE_FMT_HEX = 0x1100
PFX_FILE_FMT_ZIP = 0x2000
PFX_FILE_FMT_GZ = 0x2100
PFX_FILE_FMT_PFX = 0x3000
PFX_FILE_FMT_IMG = 0x5000
PFX_FILE_ATTR_MASK = 0x00FF
PFX_WAV_ATTR_SAMPLE_RATE_MASK = 0x0001
PFX_WAV_ATTR_SAMPLE_RATE_22K = 0x0000
PFX_WAV_ATTR_SAMPLE_RATE_11K = 0x0001
PFX_WAV_ATTR_QUANTIZATION_MASK = 0x0002
PFX_WAV_ATTR_QUANTIZATION_16 = 0x0000
PFX_WAV_ATTR_QUANTIZATION_8 = 0x0002
PFX_FILE_ATTR_SCRIPT = 0x0080
PFX_FILE_INVALID_ID = 0xFF

PFX_SOUND_ATTR_MOTOR_MASK = 0x007C
PFX_SOUND_ATTR_MOTOR_LOOP_MASK = 0x0060
PFX_SOUND_ATTR_MOTOR_LOOP_NOTCH = 0x0020
PFX_SOUND_ATTR_MOTOR_LOOP_ACCEL = 0x0040
PFX_SOUND_ATTR_MOTOR_LOOP_DECEL = 0x0060
PFX_SOUND_ATTR_MOTOR_LOOP_IDX = 0x001C
PFX_SOUND_ATTR_MOTOR_LOOP_IDX_SH = 2
PFX_SOUND_ATTR_MOTOR_LOOP_N1 = 0x0020
PFX_SOUND_ATTR_MOTOR_LOOP_N2 = 0x0024
PFX_SOUND_ATTR_MOTOR_LOOP_N3 = 0x0028
PFX_SOUND_ATTR_MOTOR_LOOP_N4 = 0x002C
PFX_SOUND_ATTR_MOTOR_LOOP_N5 = 0x0030
PFX_SOUND_ATTR_MOTOR_LOOP_N6 = 0x0034
PFX_SOUND_ATTR_MOTOR_LOOP_N7 = 0x0038
PFX_SOUND_ATTR_MOTOR_LOOP_N8 = 0x003C
PFX_SOUND_ATTR_MOTOR_ACCEL12 = 0x0040
PFX_SOUND_ATTR_MOTOR_ACCEL23 = 0x0044
PFX_SOUND_ATTR_MOTOR_ACCEL34 = 0x0048
PFX_SOUND_ATTR_MOTOR_ACCEL45 = 0x004C
PFX_SOUND_ATTR_MOTOR_ACCEL56 = 0x0050
PFX_SOUND_ATTR_MOTOR_ACCEL67 = 0x0054
PFX_SOUND_ATTR_MOTOR_ACCEL78 = 0x0058
PFX_SOUND_ATTR_MOTOR_STARTUP = 0x005C
PFX_SOUND_ATTR_MOTOR_DECEL21 = 0x0060
PFX_SOUND_ATTR_MOTOR_DECEL32 = 0x0064
PFX_SOUND_ATTR_MOTOR_DECEL43 = 0x0068
PFX_SOUND_ATTR_MOTOR_DECEL54 = 0x006C
PFX_SOUND_ATTR_MOTOR_DECEL65 = 0x0070
PFX_SOUND_ATTR_MOTOR_DECEL76 = 0x0074
PFX_SOUND_ATTR_MOTOR_DECEL87 = 0x0078
PFX_SOUND_ATTR_MOTOR_SHUTDOWN = 0x007C
PFX_NOTIFICATION_AUDIO_PLAY_DONE = 0x01
PFX_NOTIFICATION_AUDIO_PLAY = 0x02
PFX_NOTIFICATION_MOTORA_CURR_SPD = 0x04
PFX_NOTIFICATION_MOTORA_STOP = 0x08
PFX_NOTIFICATION_MOTORB_CURR_SPD = 0x10
PFX_NOTIFICATION_MOTORB_STOP = 0x20
PFX_NOTIFICATION_TO_USB = 0x80
PFX_NOTIFICATION_TO_BLE = 0x40
PFX_STATUS_NORMAL = 0x00
PFX_STATUS_NORMAL_PENDING = 0x33
PFX_STATUS_SERVICE = 0x55
PFX_STATUS_SERVICE_PENDING = 0x53
PFX_STATUS_SERVICE_BUSY = 0x5B
PFX_ERR_NONE = 0x00
PFX_ERR_VERIFY_PASS = 0x00
PFX_ERR_VERIFY_FAIL = 0x01
PFX_ERR_TRANSFER_REQUEST_OK = 0x00
PFX_ERR_TRANSFER_FILE_EXISTS = 0x02
PFX_ERR_TRANSFER_TOO_BIG = 0x03
PFX_ERR_TRANSFER_INVALID = 0x04
PFX_ERR_TRANSFER_FILE_NOT_FOUND = 0x05
PFX_ERR_TRANSFER_ERROR = 0xFF
PFX_ERR_TRANSFER_CRC_MISMATCH = 0x06
PFX_ERR_TRANSFER_BUSY_WAIT = 0x07
PFX_ERR_TRANSFER_LUT_FULL = 0x08
PFX_ERR_TRANSFER_COMPLETE = 0x09
PFX_ERR_UPGRADE_FAIL = 0x80
PFX_ERR_FILE_SYSTEM_ERR = 0xF0
PFX_ERR_FILE_INVALID = 0xF1
PFX_ERR_FILE_OUT_OF_RANGE = 0xF2
PFX_ERR_FILE_READ_ONLY = 0xF3
PFX_ERR_FILE_TOO_BIG = 0xF4
PFX_ERR_FILE_NOT_FOUND = 0xF5
PFX_ERR_FILE_NOT_UNIQUE = 0xF6
PFX_ERR_FILE_LOCKED_BUSY = 0xF7
PFX_ERR_FILE_SYSTEM_FULL = 0xF8
PFX_ERR_FILE_SYSTEM_TIMEOUT = 0xF9
PFX_ERR_FILE_INVALID_ADDRESS = 0xFA
PFX_ERR_FILE_NEXT_SECTOR = 0xFB
PFX_ERR_FILE_ACCESS_DENIED = 0xFC
PFX_ERR_FILE_EOF = 0xFF
PFX_ERR_BLE_FAULT = 0x0B
PFX_ERR_SPKR_SHORTCIR_FAULT = 0x04
PFX_ERR_DAC_OVERTEMP_FAULT = 0x08
PFX_ERR_TRAP_BROWNOUT_RST = 0x0A
PFX_ERR_TRAP_CONFLICT = 0x10
PFX_ERR_TRAP_ILLEGAL_OPCODE = 0x20
PFX_ERR_TRAP_CONFIG_MISMATCH = 0x40

# #endif /* end of include guard: _PFX_H_ */



#! /usr/bin/env python3
import argparse
import time
from datetime import datetime
from functools import update_wrapper
from time import sleep

from rich import box, print
from rich.align import Align
from rich.bar import Bar
from rich.console import Console, RenderGroup
from rich.layout import Layout
from rich.live import Live
from rich.panel import Panel
from rich.progress import BarColumn, Progress, SpinnerColumn, TextColumn
from rich.progress_bar import ProgressBar
from rich.syntax import Syntax
from rich.table import Table
from rich.text import Text

from pfxbrick import *

console = Console()


def make_layout() -> Layout:
    """Define the layout."""
    layout = Layout(name="root")
    layout.split(
        Layout(name="header", size=3),
        Layout(name="brightvol", size=3),
        Layout(name="status", size=4),
        Layout(name="filesys", size=4),
        Layout(name="bluetooth", size=4),
        Layout(name="lights", size=11),
        Layout(name="motors", size=5),
        Layout(name="motor_rate", size=5),
        Layout(name="audio", size=7),
    )
    layout["brightvol"].split_row(
        Layout(name="bright"),
        Layout(name="vol", ratio=1),
    )
    layout["status"].split_row(
        Layout(name="status1"),
        Layout(name="status2"),
    )
    layout["audio"].split_row(
        Layout(name="audio_ch"),
        Layout(name="audio_state"),
    )
    layout["audio_state"].split_column(
        Layout(name="audio_peak"),
        Layout(name="audio_idx"),
    )
    return layout


class Header:
    """Display header with clock."""

    def __init__(self, brick):
        self.brick = brick

    def __rich__(self) -> Panel:
        grid = Table.grid(expand=False)
        grid.add_column(justify="left", min_width=22, ratio=1)
        grid.add_column(justify="left", min_width=15, ratio=1)
        grid.add_column(justify="left", min_width=42, ratio=1)
        grid.add_column(justify="right", ratio=1)
        grid.add_row(
            "[light_slate_blue]%s [bold cyan]%s"
            % (self.brick.product_id, self.brick.product_desc),
            "S/N: [yellow]%s" % (self.brick.serial_no),
            "[white]Firmware [green]v.%s build %s [white]ICD [green]v.%s"
            % (self.brick.firmware_ver, self.brick.firmware_build, self.brick.icd_rev),
            "",
            "[bold yellow]%s" % (self.brick.name),
            # datetime.now().ctime().replace(":", "[blink]:[/]"),
        )
        return Panel(grid)


def update_status(brick):
    st = brick.state
    panel = Table.grid(padding=0)
    for _ in range(8):
        panel.add_column(ratio=1, min_width=7, justify="center")
    panel.add_row("State", "Err", "sec", "msec", "Script", "Line", "L2")
    es = (
        "[bold red]0x%02X" % (brick.error)
        if brick.error > 0
        else "[green]0x%02X" % (brick.error)
    )
    ss = (
        "[magenta]%02X" % (st.script_state)
        if st.script_state == 0
        else console.status("[bold magenta]0x%02X" % (st.script_state))
    )
    panel.add_row(
        "0x%02X" % (brick.status),
        es,
        "[cyan]%5d" % (st.slow_count),
        "[cyan]%5d" % (st.millisec_count),
        ss,
        "[magenta]%3d" % (st.script_line),
        "%02X" % (st.status_latch2),
    )
    return Panel(panel)


def update_fs_status(st):
    panel = Table.grid(padding=0)
    for i in range(9):
        width = 7 if i < 6 else 19
        panel.add_column(ratio=1, min_width=width, justify="center")
    panel.add_row(
        "", "State", "Flags", "Files", "Open", "Erase", "Capacity", "Free", "Empty"
    )
    panel.add_row(
        "[bold white]Filesystem ",
        "[cyan]0x%02X" % (st.filesys.task_state),
        "[cyan]0x%02X" % (st.filesys.flags),
        "[magenta]%2d" % (st.filesys.file_count),
        "[magenta]%2d" % (st.filesys.open_files),
        "[cyan]0x%04X" % (st.filesys.erase_sector),
        "%d sec [b]%d kB[/b]"
        % (st.filesys.sector_capacity, st.filesys.sector_capacity * 4096 / 1000),
        "%d sec [b]%d kB[/b]"
        % (st.filesys.free_sectors, st.filesys.free_sectors * 4096 / 1000),
        "%d sec [b]%d kB[/b]"
        % (st.filesys.empty_sectors, st.filesys.empty_sectors * 4096 / 1000),
    )
    return Panel(panel)


def update_bt_status(st):
    panel = Table.grid(padding=0)
    for i in range(9):
        justify = "center" if i < 7 else "right"
        panel.add_column(ratio=1, min_width=9, justify=justify)
    panel.add_row(
        "", "State", "Flags", "Error", "Services", "Features", "Auth", "Tx", "Rx"
    )
    es = (
        "[bold red]0x%04X" % (st.bt.error)
        if st.bt.error > 0
        else "[green]0x%04X" % (st.bt.error)
    )
    stx = "[orange1]%d" % (st.bt.tx_count)
    srx = "[green]%d" % (st.bt.rx_count)
    if st.status_latch1 & 0x40:
        stx = console.status(stx)
        srx = console.status(srx)

    panel.add_row(
        "[bold white]Bluetooth ",
        "[cyan]0x%02X" % (st.bt.state),
        "[cyan]0x%04X" % (st.bt.flags),
        es,
        "0x%04X" % (st.bt.services),
        "0x%04X" % (st.bt.features),
        "0x%04X" % (st.bt.auth),
        stx,
        srx,
    )
    return Panel(panel)


def update_audio(st, brick):
    panel = Table.grid(expand=True)
    panel.title = "Audio Channels"
    panel.title_style = "bold white"
    panel.add_column(ratio=1, justify="left", min_width=4)
    panel.add_column(ratio=2, justify="left", min_width=10)
    panel.add_column(ratio=5, min_width=24)
    for i in range(4):
        fid = st.audio_ch[i].file_id
        fn = ""
        if not fid == 0xFF:
            fn = brick.filedir.get_file_dir_entry(fid)
            if fn is not None:
                fn = fn.name
        fs = "File: --- " if fid == 0xFF else "File: [aquamarine3]0x%02X %s" % (fid, fn)
        panel.add_row(
            "Ch %d " % (i),
            "Mode: [cyan]0x%02X " % (st.audio_ch[i].mode),
            fs,
        )
    return Panel(panel)


def update_audio_state(st):
    panel = Table.grid()
    level_bar = ProgressBar(
        total=255,
        complete_style="aquamarine3",
        finished_style="aquamarine3",
        completed=st.audio_peak,
    )
    for i in range(3):
        panel.add_column(ratio=1)
    panel.add_row(
        "Audio peak: ",
        level_bar,
        "[aquamarine3]0x%02X" % (st.audio_peak),
    )
    return Panel(panel)


def update_audio_idx(brick):
    panel = Table.grid()
    panel.add_column(ratio=2)
    for _ in range(brick.config.settings.notchCount):
        panel.add_column(ratio=1)
    notches = ["0x%02X " % x for x in brick.config.settings.notchBounds]
    n = brick.state.audio_notch
    if n < len(brick.config.settings.notchBounds):
        notches[n] = "[black on white]0x%02X " % (brick.config.settings.notchBounds[n])
    panel.add_row("Curr Notch ", "%d" % (brick.state.audio_notch + 1))
    panel.add_row("Notch", *notches)
    return Panel(panel)


def update_brightness(value) -> Panel:
    """Some example content."""
    panel = Table.grid(padding=1)
    bright_bar = ProgressBar(
        total=255,
        complete_style="light_slate_blue",
        finished_style="light_slate_blue",
        completed=value,
    )
    panel.add_column(ratio=1)
    panel.add_column(ratio=1)
    panel.add_column(ratio=1)
    panel.add_row("Brightness: ", bright_bar, "[light_slate_blue]%3d" % (value))
    return Panel(panel)


def update_volume(value) -> Panel:
    """Some example content."""
    panel = Table.grid()
    bright_bar = ProgressBar(
        total=255,
        complete_style="aquamarine3",
        finished_style="aquamarine3",
        completed=value,
    )
    panel.add_column(ratio=1)
    panel.add_column(ratio=1)
    panel.add_column(ratio=1)
    panel.add_row("Volume: ", bright_bar, "[aquamarine3]%3d" % (value))
    return Panel(panel)


def update_motors(st):
    panel = Table.grid()
    panel.title = "Motor Channels"
    panel.title_style = "bold white"
    panel.add_column(ratio=1)
    panel.add_column(ratio=1)
    panel.add_column(ratio=1)
    panel.add_column(ratio=1)
    panel.add_column(ratio=1)
    panel.add_column(ratio=1)
    panel.add_column(ratio=1)
    panel.add_column(ratio=1)
    panel.add_column(ratio=1)
    panel.add_column(ratio=1)
    for ch in range(0, 2):
        motor = st.motors[ch]
        tgtspd = motor.target_speed
        curspd = motor.current_speed
        pwmspd = motor.pwm_speed
        chactive = (ch == 0 and st.status_latch2 & 0x03) or (
            ch == 1 and st.status_latch2 & 0x0C
        )
        style = "green"
        if not chactive:
            s = ":black_medium_square:"
        if chactive and motor.dir == "Forward":
            s = ":arrow_forward:"
            style = "green"
        elif chactive:
            s = ":arrow_backward:"
            tgtspd = (~tgtspd + 1) & 0xFF
            curspd = (~curspd + 1) & 0xFF
            style = "red"
        tgt = ProgressBar(
            total=255, completed=tgtspd, complete_style=style, finished_style=style
        )
        curr = ProgressBar(
            total=255, completed=curspd, complete_style=style, finished_style=style
        )
        pwm = ProgressBar(
            total=255, completed=pwmspd, complete_style=style, finished_style=style
        )
        panel.add_row(
            "Ch %d %s Tgt: " % (ch + 1, s),
            tgt,
            " [cyan]%3d " % (tgtspd),
            "Curr: ",
            curr,
            " [cyan]%3d " % (curspd),
            "PWM: ",
            pwm,
            " [cyan]0x%02X" % (pwmspd),
        )
    return Panel(panel)


def update_lights(st):
    panel = Table.grid()
    panel.title = "Light Channels"
    panel.title_style = "bold white"
    panel.add_column(ratio=1)
    panel.add_column(ratio=1)
    panel.add_column(ratio=1)
    panel.add_column(ratio=1)
    panel.add_column(ratio=1)
    panel.add_column(ratio=1)
    panel.add_column(ratio=1)
    style = "light_goldenrod3"
    for ch in range(0, 8):
        light = st.lights[ch]
        tgt = ProgressBar(
            total=255,
            completed=light.target_level,
            complete_style=style,
            finished_style=style,
        )
        curr = ProgressBar(
            total=255,
            completed=light.current_level,
            complete_style=style,
            finished_style=style,
        )
        if light.active:
            s = ":black_square_button: "
        else:
            s = ":black_large_square: "
        panel.add_row(
            "Ch %d %s Tgt: " % (ch + 1, s),
            tgt,
            " [cyan]%3d " % (light.target_level),
            "Curr: ",
            curr,
            " [cyan]%3d " % (light.current_level),
        )
    return Panel(panel)


def update_bitfield(byte, size, labels):
    panel = Table.grid(expand=True)
    for _ in range(size):
        panel.add_column(width=8, justify="center", ratio=1)
    bits = []
    for x in range(size):
        mask = 1 << x
        if byte & mask:
            s = " :black_square_button: "
        else:
            s = " :black_large_square: "
        bits.append(s)
    panel.add_row(*bits)
    panel.add_row(*labels)
    return Panel(panel)


def update_motor_rates(st, b):
    panel = Table.grid(expand=True)
    panel.title = "Triggered Sound State"
    panel.title_style = "bold white"
    for _ in range(8):
        panel.add_column(width=8, justify="center", ratio=1)
    panel.add_row(
        "Motor Spd",
        "Motor PWM",
        "Motor Rate",
        "[Change Dir]",
        "[Set Off]",
        "[Rapid Acc]",
        "[Rapid Dec]",
        "[Brake]",
    )
    if st.motor_rate_ptr & 0x80:
        rate = -((~st.motor_rate_ptr + 1) & 0xFF)
    else:
        rate = st.motor_rate_ptr
    if st.trig_change_dir_state > 0:
        s1 = "[bold magenta]%2d" % (st.trig_change_dir_state)
    else:
        s1 = "[bold black]%2d" % (st.trig_change_dir_state)
    if st.trig_set_off_state > 0:
        s2 = "[bold magenta]%2d" % (st.trig_set_off_state)
    else:
        s2 = "[bold black]%2d" % (st.trig_set_off_state)
    if st.trig_rapid_accel_state > 0:
        s3 = "[bold magenta]%2d" % (st.trig_rapid_accel_state)
    else:
        s3 = "[bold black]%2d" % (st.trig_rapid_accel_state)
    if st.trig_rapid_decel_state > 0:
        s4 = "[bold magenta]%2d" % (st.trig_rapid_decel_state)
    else:
        s4 = "[bold black]%2d" % (st.trig_rapid_decel_state)
    if st.trig_brake_state > 0:
        s5 = "[bold magenta]%2d" % (st.trig_brake_state)
    else:
        s5 = "[bold black]%2d" % (st.trig_brake_state)
    panel.add_row(
        "[cyan]%3d" % (st.motor_ptr),
        "[cyan]0x%02X" % (st.motor_pwm_ptr),
        "[green]%2d" % (rate),
        s1,
        s2,
        "[green]%2d %s" % (b.config.settings.rapidAccelThr, s3),
        "[green]%2d %s" % (b.config.settings.rapidDecelThr, s4),
        "[green]%2d [cyan]%2d %s"
        % (
            b.config.settings.brakeDecelThr,
            b.config.settings.brakeSpeedThr,
            s5,
        ),
    )
    return Panel(panel)


def main():
    parser = argparse.ArgumentParser(
        description="PFx Brick real time monitoring utility. Press <Ctrl>-C to exit monitor."
    )
    parser.add_argument(
        "-s",
        "--serialno",
        default=None,
        help="Perform monitoring on PFx Brick with specified serial number",
    )
    args = parser.parse_args()
    argsd = vars(args)

    b = get_one_pfxbrick(argsd["serialno"])
    r = b.open()
    if not r:
        exit()
    b.get_status()
    b.get_config()
    icd = b.get_icd_rev()
    name = b.get_name()
    b.refresh_file_dir()

    layout = make_layout()
    layout["header"].update(Header(b))
    with Live(layout, refresh_per_second=8, screen=True):
        flip = True
        while True:
            st = b.get_current_state()
            b.get_status()
            if flip:
                b.get_fs_state()
                flip = False
            else:
                if b.has_bluetooth:
                    b.get_bt_state()
                flip = True
            layout["bright"].update(update_brightness(st.brightness))
            layout["vol"].update(update_volume(st.volume))
            layout["status1"].update(
                update_bitfield(
                    st.status_latch1,
                    8,
                    [
                        "USB:link:",
                        "USB",
                        "IR",
                        "IR:lock:",
                        ":speaker:",
                        "BLE:link:",
                        "BLE",
                        "FS",
                    ],
                )
            )
            layout["filesys"].update(update_fs_status(st))
            layout["status2"].update(update_status(b))
            if b.has_bluetooth:
                layout["bluetooth"].update(update_bt_status(st))
            layout["lights"].update(update_lights(st))
            layout["motors"].update(update_motors(st))
            layout["motor_rate"].update(update_motor_rates(st, b))
            layout["audio_ch"].update(update_audio(st, b))
            layout["audio_peak"].update(update_audio_state(st))
            layout["audio_idx"].update(update_audio_idx(b))

            time.sleep(0.15)


if __name__ == "__main__":
    main()



#! /usr/bin/env python3
"""
pfxplay - play an audio file on the PFx Brick
"""
import argparse

from pfxbrick import *


def main():
    parser = argparse.ArgumentParser(
        description="Play an audio file on the PFx Brick",
        prefix_chars="-+",
    )
    parser.add_argument(
        "file", metavar="file", type=str, help="file name or file ID to play"
    )
    parser.add_argument(
        "-l",
        "--loop",
        action="store_true",
        default=False,
        help="loop playback repeatedly",
    )
    parser.add_argument(
        "-e",
        "--end",
        action="store_true",
        default=False,
        help="end playback",
    )
    parser.add_argument(
        "-s",
        "--serialno",
        default=None,
        help="Specify PFx Brick with serial number (if more than one connected)",
    )
    args = parser.parse_args()
    argsd = vars(args)

    b = get_one_pfxbrick(argsd["serialno"])
    r = b.open()
    if not r:
        exit()
    b.refresh_file_dir()
    f = str(argsd["file"])
    if f.isnumeric():
        f = int(argsd["file"])
    fid = b.file_id_from_str_or_int(f)
    if fid < 0xFF:
        fd = b.filedir.get_file_dir_entry(fid)
        if fd.is_audio_file():
            if argsd["loop"]:
                b.repeat_audio_file(fid)
            elif argsd["end"]:
                b.stop_audio_file(fid)
            else:
                b.play_audio_file(fid)
        else:
            print("File %s is not an audio file" % (argsd["file"]))
    else:
        print("File %s is not found on the PFx Brick" % (argsd["file"]))
    b.close()


if __name__ == "__main__":
    main()



#! /usr/bin/env python3
import argparse

from pfxbrick import *


def main():
    parser = argparse.ArgumentParser(
        description="copy a file from the PFx Brick to host computer",
        prefix_chars="-+",
    )
    parser.add_argument(
        "file", metavar="file", type=str, help="is file ID or filename to copy"
    )
    parser.add_argument(
        "dest",
        type=str,
        default=None,
        nargs="?",
        help="is optional local file path override for copied file",
    )
    parser.add_argument(
        "-s",
        "--serialno",
        default=None,
        help="Specify PFx Brick with serial number (if more than one connected)",
    )
    args = parser.parse_args()
    argsd = vars(args)

    b = get_one_pfxbrick(argsd["serialno"])
    r = b.open()
    if not r:
        exit()
    b.open()
    b.refresh_file_dir()
    f = str(argsd["file"])
    if f.isnumeric():
        f = int(argsd["file"])
    fid = b.file_id_from_str_or_int(f)
    fd = b.filedir.get_file_dir_entry(fid)
    if argsd["dest"] is not None:
        fn = argsd["dest"]
    else:
        fn = fd.name
        if fd.is_audio_file() and not fd.name.lower().endswith(".wav"):
            fn = fn + ".wav"
        if fd.is_script_file() and not fd.name.lower().endswith(".pfx"):
            fn = fn + ".pfx"
    print("Copying file %s as %s from brick..." % (f, fn))
    b.get_file(fid, fn)
    b.close()


if __name__ == "__main__":
    main()



#! /usr/bin/env python3
"""
pfxfat - shows the PFx Brick file allocation table (FAT)
"""
import argparse
from sys import argv

from rich import print

from pfxbrick import *


def main():
    parser = argparse.ArgumentParser(
        description="Dumps the contents of the PFx Brick file allocation table (FAT)",
        prefix_chars="-+",
    )
    parser.add_argument(
        "-s",
        "--serialno",
        default=None,
        help="Specify PFx Brick with serial number (if more than one connected)",
    )
    args = parser.parse_args()
    argsd = vars(args)

    b = get_one_pfxbrick(argsd["serialno"])
    r = b.open()
    if not r:
        exit()
    rb = flash_read(b, 0xFFC000, 8192)
    b.close()
    x = []
    for i in range(0, len(rb) - 1, 2):
        x.append(rb[i + 1])
        x.append(rb[i])

    a = 0
    s = []
    ads = "[bold white]%06X[/] " % (a)
    s.append("%s " % (ads))
    for i, b in enumerate(x):
        if (i % 2) == 0:
            if x[i] == 0xFF and x[i + 1] == 0xFF:
                cs = "[bold red]"
            elif x[i] == 0xFF and x[i + 1] == 0xFE:
                cs = "[bold magenta]"
            elif x[i] == 0xFF and x[i + 1] == 0xF3:
                cs = "[green]"
            elif x[i] == 0xFF and x[i + 1] == 0xF1:
                cs = "[bold orange3]"
            elif x[i] == 0xFF and x[i + 1] == 0xF7:
                cs = "[bold yellow]"
            elif i > 1:
                if abs(int(x[i + 1]) - int(x[i - 1])) > 1:
                    cs = "[bold cyan]"
                else:
                    cs = "[bold white]"
            else:
                cs = "[bold white]"
        s.append("%s%02X[/]" % (cs, b))
        if (i + 1) % 2 == 0 and i > 0:
            s.append(" ")
        if (i + 1) % 16 == 0 and i > 0:
            s.append(" ")
        if (i + 1) % 32 == 0 and i > 0:
            a += 16
            ads = "[bold white]%06X[/] " % (a)
            s.append("\n")
            s.append("%s " % (ads))
    nb = len(x) % 32
    if nb:
        for i in range(32 - nb + 1):
            s.append("   ")
    else:
        s.pop()
    print("".join(s))


if __name__ == "__main__":
    main()
